// Bessel.cpp

#include "Eclipse.h"

// =========================================================================
// Function to calculate Besselian elements of solar eclipse.
// t is Julian day number. x, y, l1, l2 in units of Earth equatorial radius.
// =========================================================================
void GetBesselElements (double t, double& x, double& y, double& d, double& f1, double& f2, double& l1, double& l2, double& mu)
{
  // Get equatorial coordinates of Sun
  double r, delta, RA;
  SunEquatorial (t, r, delta, RA, 1, 1, 1, 1);
  r *= AU /Re;

  // Get equatorial coordinates of Moon
  double r1, delta1, RA1;
  MoonEquatorial (t, r1, delta1, RA1, 1, 1, 1, 1);
  r1 *= AU /Re;

  // Calculate a and d
  double sind  = sin (delta),  cosd  = cos (delta),  sinr  = sin (RA),  cosr  = cos (RA);
  double sind1 = sin (delta1), cosd1 = cos (delta1), sinr1 = sin (RA1), cosr1 = cos (RA1);

  double X = r * cosd * cosr - r1 * cosd1 * cosr1;
  double Y = r * cosd * sinr - r1 * cosd1 * sinr1;
  double Z = r * sind - r1 * sind1;
  double R = sqrt (X*X + Y*Y + Z*Z);

  double a = atan2 (Y, X);
  d = asin (Z /R);
  
  // Calculate x, y, z, z1
  double sinD = sin (d), cosD = cos (d), z, z1;
  x  = r1 * cosd1 * sin (RA1 - a);
  y  = r1 * (sind1 * cosD - cosd1 * sinD * cos (RA1 - a));
  z  = r  * (sind  * sinD + cosd  * cosD * cos (RA  - a));
  z1 = r1 * (sind1 * sinD + cosd1 * cosD * cos (RA1 - a));

  // Calculate f1, f2
  f1 = asin ((Rs + k1) /(z - z1));
  f2 = asin ((Rs - k2) /(z - z1));

  // Calculate l1, l2
  l1 = z1 * tan (f1) + k1 /cos (f1);
  l2 = z1 * tan (f2) - k2 /cos (f2);

  // Calculate mean Greenwich sidereal time (in radians)
  double theta = MeanGreenwichSiderealTime (t);
  
  // Correct sidereal time for nutation
  double Dpsi, eps;
  NutationLongitude (t, Dpsi);
  Obliquity (t, eps);
  theta += Dpsi * cos (eps);

  // Calculate mu
  mu = theta - a;
  mu = atan2 (sin (mu), cos (mu));
}

// =======================================================
// Function to calculate derivatives of Besselian elements
// =======================================================
void GetBesselElementDerivatives (double t, double &xdot, double &ydot, double &mudot, double &ddot, double &l1dot, double &l2dot)
{
  // dt is 1 second
  double dt = 1./24./60./60.;
  
  // Get Besselian elements at t1 = t - dt
  double t1 = t - dt;
  double x1, y1, d1, f11, f21, l11, l21, mu1;
  GetBesselElements (t1, x1, y1, d1, f11, f21, l11, l21, mu1);
  
  // Get Besselian elements at t2 = t + dt
  double t2 = t + dt;
  double x2, y2, d2, f12, f22, l12, l22, mu2;
  GetBesselElements (t2, x2, y2, d2, f12, f22, l12, l22, mu2);
  
  // Calculate derivatives
  xdot  = (x2 - x1)   /(t2 - t1);
  ydot  = (y2 - y1)   /(t2 - t1);
  mudot = (mu2 - mu1) /(t2 - t1);
  ddot  = (d2  - d1)  /(t2 - t1);
  l1dot = (l12 - l11) /(t2 - t1);
  l2dot = (l22 - l21) /(t2 - t1);
}

// ==================================================================
// Function to produce table of Besselian elements for solar eclipse.
// Y, M, D are year, month, and day of central time.
// ==================================================================
void WriteBesselElementsTable (int Year, int Month, int Day, int Hour, double tanf1, double tanf2, double mudot, char filename[])
{
  FILE *file = OpenFile (filename, "w");
  
  printf ("\nBesselian Elements:\n\n");
  printf ("TDT       x        y           x'         y'           sin(d)   cos(d)        l1       l2            mu\n");
  fprintf (file, "Besselian Elements:\n\n");
  fprintf (file, "TDT       x        y           x'         y'           sin(d)   cos(d)        l1       l2            mu\n");
  
  for (int i = -5; i <= +5; i++)
    {
      // Calculate table dates
      double D = double (Day) + double (Hour + i) /24.;
      double t;
      DatetoJulian (Year, Month, D, t);
      
      // Get Besselian elements
      double x, y, d, f1, f2, l1, l2, mu;
      GetBesselElements (t, x, y, d, f1, f2, l1, l2, mu);
      double gamma = sqrt (x*x + y*y);
      
      // Calculate elements of mu
      mu *= (180./M_PI);
      if (mu < 0.)
	mu += 360.;

      int    dd = int (mu);
      double MM = 60. * (mu - double (dd));
      int    mm = int (MM);
      double SS = 60. * (MM - double (mm));
      int    ss = rint (SS);
      if (ss == 60)
	{
	  mm += 1;
	  ss  = 0;
	}

      // Calculate derivatives of Besselian elements
      double xdot, ydot, mudot, ddot, l1dot, l2dot;
      GetBesselElementDerivatives (t, xdot, ydot, mudot, ddot, l1dot, l2dot);
      xdot /= 60.*24.;
      ydot /= 60.*24.;
            
      // Output Besselian element data
      if (fabs (gamma - l1) < 1.1)
	{
	  printf ("%2d :: %+8.5f %+8.5f ::  %9.7f %+9.7f ::  %+8.5f %8.5f ::  %8.5f %+8.5f ::  %3d:%02d:%02d\n",
		  Hour+i, x, y, xdot, ydot, sin (d), cos (d), l1, l2, dd, mm, ss);
	  fprintf (file, "%2d :: %+8.5f %+8.5f ::  %9.7f %+9.7f ::  %+8.5f %8.5f ::  %8.5f %+8.5f ::  %3d:%02d:%02d\n",
		   Hour+i, x, y, xdot, ydot, sin (d), cos (d), l1, l2, dd, mm, ss);
	}
    }

  // Output common Besselian element data
  mudot *= (180./M_PI) * 60. /60./24.;

  printf ("\n");
  printf ("sin(f1) = %8.6f  sin(f2) = %8.6f ::  mu' = %7.4f\n", tanf1, tanf2, mudot);
  printf ("\n");

  fprintf (file, "\n");
  fprintf (file, "sin(f1) = %8.6f  sin(f2) = %8.6f ::  mu' = %7.4f\n", tanf1, tanf2, mudot);

  fclose (file);
}
