// Delta.cpp

#include "Eclipse.h"

// ==============================
// Function to calculate gammaDot
// ==============================
double GetgammaDot (double t)
{
  double x, y, d, f1, f2, l1, l2, mu;
  GetBesselElements (t, x, y, d, f1, f2, l1, l2, mu);

  double xdot, ydot, mudot, ddot, l1dot, l2dot;
  GetBesselElementDerivatives (t, xdot, ydot, mudot, ddot, l1dot, l2dot); 

  return 2.*x*xdot + 2.*y*ydot;
}

// ===========================
// Function to calculate Delta
// ===========================
double GetDelta (double t)
{
  double x, y, d, f1, f2, l1, l2, mu;
  GetBesselElements (t, x, y, d, f1, f2, l1, l2, mu);

  double omega = 1. /sqrt (1. - fe * (2. - fe) * cos (d) * cos (d));
  double y1    = omega * y;
  
  return 1. - x*x - y1*y1;
}

// ===========================
// Function to calculate Delta
// ===========================
double GetDelta (double x, double y, double d, double l, double f, double alpha)
{
  // Iterate to take into account variation of L with zeta
  double xi, eta, zeta = 0., L, omega;
  omega = 1. /sqrt (1. - fe * (2. - fe) * cos (d) * cos (d));
  for (int i = 0; i < MAX; i++)
    {
      L   = l - zeta * tan (f);
      xi  = x + fabs (L) * cos (alpha);
      eta = y + fabs (L) * sin (alpha);
      GetZeta0 (xi, eta, d, omega, zeta);
    }
  
  return 1. - xi*xi - omega*omega*eta*eta;
}

// =================================
// Function to calculate DeltaDotDot
// =================================
double GetDeltaDotDot (double t)
{
  // dt is 1 minute
  double dt = 1./24./60.;
  
  double t0 = t;
  double D0 = GetDelta (t0);
  
  double t1 = t - dt;
  double D1 = GetDelta (t1);

  double t2 = t + dt;
  double D2 = GetDelta (t2);

  return (D1 - 2.*D0 + D2) /dt/dt;
}

// ===============================
// Function to calculate Delta_max
//
// flag = 1/0 for penumbra/umbra
//
// ===============================
double GetDelta_max (double t, int flag)
{
  double x, y, d, f1, f2, l1, l2, mu;
  GetBesselElements (t, x, y, d, f1, f2, l1, l2, mu);
  
  double l, f;
  if (flag)
    {
      l = l1;
      f = f1;
    }
  else
    {
      l = l2;
      f = f2;
    }

  // Iterate to take into account variation of L with zeta
  double zeta = 0., L, alpha, xi, eta, omega;
  omega = 1. /sqrt (1. - fe * (2. - fe) * cos (d) * cos (d));
  for (int i = 0; i < MAX; i++)
    {
      L     = l - zeta * tan (f);
      alpha = Getalpha_max (x, y, d, L);
      xi    = x + fabs (L) * cos (alpha);
      eta   = y + fabs (L) * sin (alpha);
      GetZeta0 (xi, eta, d, omega, zeta);
    }
  
  return 1. - xi*xi - omega*omega*eta*eta;
}

// ===============================
// Function to calculate Delta_min
//
// flag = 1/0 for penumbra/umbra
//
// ===============================
double GetDelta_min (double t, int flag)
{
  double x, y, d, f1, f2, l1, l2, mu;
  GetBesselElements (t, x, y, d, f1, f2, l1, l2, mu);
  
  double l, f;
  if (flag)
    {
      l = l1;
      f = f1;
    }
  else
    {
      l = l2;
      f = f2;
    }

  // Iterate to take into account variation of L with zeta
  double zeta = 0., L, alpha, xi, eta, omega;
  omega = 1. /sqrt (1. - fe * (2. - fe) * cos (d) * cos (d));
  for (int i = 0; i < MAX; i++)
    {
      L     = l - zeta * tan (f);
      alpha = Getalpha_min (x, y, d, L);
      xi    = x + fabs (L) * cos (alpha);
      eta   = y + fabs (L) * sin (alpha);
      GetZeta0 (xi, eta, d, omega, zeta);
    }
  
  return 1. - xi*xi - omega*omega*eta*eta;
}

// ===============================
// Function to calculate alpha_max
// ===============================
double Getalpha_max (double x, double y, double d, double L)
{
  double omega = 1. /sqrt (1. - fe * (2. - fe) * cos (d) * cos (d));
  double r     = sqrt (x*x + y*y);
  double theta = atan2 (y, x);

  // Iterate
  double Omega, alpha = theta + M_PI;
  for (int i = 0; i < MAX; i++)
    {
      Omega = (omega*omega - 1.) * (sin (theta) + (fabs (L) /r) * sin (alpha)) * cos (alpha);
      alpha = theta + M_PI - asin (Omega);
    }

  return alpha;
}

// ===============================
// Function to calculate alpha_min
// ===============================
double Getalpha_min (double x, double y, double d, double L)
{
  double omega = 1. /sqrt (1. - fe * (2. - fe) * cos (d) * cos (d));
  double r     = sqrt (x*x + y*y);
  double theta = atan2 (y, x);

  // Iterate
  double Omega, alpha = theta;
  for (int i = 0; i < MAX; i++)
    {
      Omega = (omega*omega - 1.) * (sin (theta) + (fabs (L) /r) * sin (alpha)) * cos (alpha);
      alpha = theta + asin (Omega);
    }

  return alpha;
}

// ===========================
// Function to calculate alpha
// ===========================
void GetAlpha (double alpha_min, double alpha_max, double x, double y, double d, double l, double f, double &alpha)
{
  // Search for zero of Delta via bisection/Regula Falsi method
  double a0 = alpha_min;
  double a1 = alpha_max;
  if (GetDelta (x, y, d, l, f, a0) * GetDelta (x, y, d, l, f, a1) > 0.)
    {
      printf ("Fatal Error: No zero in range in GetAlpha\n");
      printf ("alpha_min = %11.4e  Delta = %11.4e\n", a0, GetDelta (x, y, d, l, f, a0));
      printf ("alpha_max = %11.4e  Delta = %11.4e\n", a1, GetDelta (x, y, d, l, f, a1));
      exit (1);
    }
  double a2 = (a0 + a1) /2.;

  double a3; int count = 0;
  do
    {
      if (GetDelta (x, y, d, l, f, a0) * GetDelta (x, y, d, l, f, a2) < 0.)
	a1 = a2;
      else
	a0 = a2;

      if (count < 10)
	a3 = (a0 + a1) /2.;
      else
	a3 = a0 - ((a1 - a0) / (GetDelta (x, y, d, l, f, a1) - GetDelta (x, y, d, l, f, a0))) * GetDelta (x, y, d, l, f, a0);

      if (fabs (a3 - a2) < ERR)
	break;

      if (count > MAXITER)
	 {
	   printf ("Warning: Root finding failed to converge in GetAlpha\n");
	   break;
	 }

      a2 = a3;
      count++;
    }
  while (1);

  alpha = a3;
}
