/************************************************************************
LUNAR SOLUTION ELP2000-82B
by Chapront-Touze M., Chapront J.
ftp://ftp.imcce.fr/pub/ephem/moon/elp82b

Copyright (c) 2005 Johannes Gajdosik

Permission is hereby granted, free of charge, to any person obtaining a
copy of this software and associated documentation files (the "Software"),
to deal in the Software without restriction, including without limitation
the rights to use, copy, modify, merge, publish, distribute, sublicense,
and/or sell copies of the Software, and to permit persons to whom the
Software is furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included
in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
****************************************************************/

#include <math.h>
#include <string.h>

static const int elp82b_max_lambda_factor[17] =
  {
    10,
    6,
    8,
    6,
    2,
    8,
    4,
    6,
    5,
    17,
    61,
    65,
    71,
    14,
    12,
    4,
    4,
  };

/* 
   ELP82B constants
*/
static const double elp82b_constants[9] =
  {
    0.000000000000000000e+00,
    0.000000000000000000e+00,
    3.850005584468033630e+05,
    -1.100000000000000039e-04,
    0.000000000000000000e+00,
    2.059999999999999765e-03,
    0.000000000000000000e+00,
    0.000000000000000000e+00,
    0.000000000000000000e+00,
  };

/*
  Delaunay's arguments (radians /Julian century^(n)):
   D^(1), l'^(1), l^(1), F^(1),
   D^(2), l'^(2), l^(2), F^(2),
   D^(3), l'^(3), l^(3), F^(3),
   D^(4), l'^(4), l^(4), F^(4)
*/
static const double del[16] =
  {
    (1732559343.73604 - 129597742.2758) * (M_PI/(180.*3600.)),
    ( 129597742.2758  -      1161.2283) * (M_PI/(180.*3600.)),
    (1732559343.73604 -  14643420.2632) * (M_PI/(180.*3600.)),
    (1732559343.73604 -  -6967919.3622) * (M_PI/(180.*3600.)),
    (-5.8883   -   -0.0202)             * (M_PI/(180.*3600.)),
    (-0.0202   -    0.5327)             * (M_PI/(180.*3600.)),
    (-5.8883   -  -38.2776)             * (M_PI/(180.*3600.)),
    (-5.8883   -    6.3622)             * (M_PI/(180.*3600.)),
    (0.006604  -      9e-6)             * (M_PI/(180.*3600.)),
    (9e-6      -  -1.38e-4)             * (M_PI/(180.*3600.)),
    (0.006604  - -0.045047)             * (M_PI/(180.*3600.)),
    (0.006604  -  0.007625)             * (M_PI/(180.*3600.)),
    (-3.169e-5 -    1.5e-7)             * (M_PI/(180.*3600.)),
    (1.5e-7    -       0.0)             * (M_PI/(180.*3600.)),
    (-3.169e-5 - 2.1301e-4)             * (M_PI/(180.*3600.)),
    (-3.169e-5 - -3.586e-5)             * (M_PI/(180.*3600.))
  };

/* 
   Precession (radians/Julian century):
    zeta^(1) = W_1^(1) + p
*/
static const double zeta[1] =
  {
    (1732559343.73604 + 5029.0966) * (M_PI/(180*3600))  
  };

/* 
   Planetary arguments (radians / Julian century):
   M_e^(1), V^(1), T^(1), M_a^(1), J^(1), S^(1), U^(1), N^(1)
*/
static const double p[8] =
  {
    538101628.68898 * (M_PI/(180.*3600.)),
    210664136.43355 * (M_PI/(180.*3600.)),
    129597742.2758  * (M_PI/(180.*3600.)),
    68905077.59284  * (M_PI/(180.*3600.)),
    10925660.42861  * (M_PI/(180.*3600.)),
    4399609.65932   * (M_PI/(180.*3600.)),
    1542481.19393   * (M_PI/(180.*3600.)),
    786550.32074    * (M_PI/(180.*3600.))
  };

/* 
   W_1^(0), W_1^(1), W_1^(2), W_1^(3), W_1^(4)
*/
static const double w[5] =
  {
    218*3600 + 18*60 + 59.95571,
    1732559343.73604,
    -5.8883,
    0.006604,
    -3.169e-5,
  };

static const double a0_div_ath_times_au =
  384747.9806448954 / (384747.9806743165 * 149597870.691);

/* 
   Data for transformation matrix
*/
static const double p1 =  1.0180391e-5;
static const double p2 =  4.7020439e-7;
static const double p3 = -5.417367e-10;
static const double p4 = -2.507948e-12;
static const double p5 =  4.63486e-15;
static const double q1 = -1.13469002e-4;
static const double q2 =  1.2372674e-7;
static const double q3 =  1.265417e-9;
static const double q4 = -1.371808e-12;
static const double q5 = -3.20334e-15;
