/************************************************************************
LUNAR SOLUTION ELP2000-82B
by Chapront-Touze M., Chapront J.
ftp://ftp.imcce.fr/pub/ephem/moon/elp82b

Copyright (c) 2005 Johannes Gajdosik

Permission is hereby granted, free of charge, to any person obtaining a
copy of this software and associated documentation files (the "Software"),
to deal in the Software without restriction, including without limitation
the rights to use, copy, modify, merge, publish, distribute, sublicense,
and/or sell copies of the Software, and to permit persons to whom the
Software is furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included
in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.

This implementation of ELP2000-82B has the following modifications compared to
the original Fortran code:

1) The series has been fundamentally rearranged into optimized instructions
    for fast calculation of the results.
2) Units are: Julian day, AU.

****************************************************************/

#include "Coefficients.h"
#include "Instructions.h"
#include "Data.h"

/* 
   Initialize cos_sin_lambda:
   (cos,sin)(1*lambda[0]),(cos,sin)(-1*lambda[0]),(cos,sin)(2*lambda[0]),...
   (cos,sin)(1*lambda[1]),(cos,sin)(-1*lambda[1]),(cos,sin)(2*lambda[1]),...
*/
static void PrepareElp82bLambdaArray (int nr_of_lambdas,
				      const int max_lambda_factor[],
				      const double lambda[],
				      double cos_sin_lambda[])
{
  int i, m;

  for (i = 0; i < nr_of_lambdas; i++)
    {
      const int max_factor = max_lambda_factor[i];
      double *cslp = cos_sin_lambda;
      cslp[0] = cos(lambda[i]);
      cslp[1] = sin(lambda[i]);
      cslp[2] =  cslp[0];
      cslp[3] = -cslp[1];
        
      for (m = 2; m <= max_factor; m++)
        {
	  /* Addition theorem:
	     cos(m*l) = cos(m0*l+m1*l) = cos(m0*l)*cos(m1*l)-sin(m0*l)*sin(m1*l)
             sin(m*l) = sin(m0*l+m1*l) = cos(m0*l)*sin(m1*l)+sin(m0*l)*cos(m1*l)
	  */

	  const double *m0 = cos_sin_lambda + ((((m+0)>>1)-1)<<2);
	  const double *m1 = cos_sin_lambda + ((((m+1)>>1)-1)<<2);
	  cslp += 4;
	  cslp[0] = m0[0] * m1[0] - m0[1] * m1[1];
	  cslp[1] = m0[0] * m1[1] + m0[1] * m1[0];
	  cslp[2] =  cslp[0];
	  cslp[3] = -cslp[1];
        }
      
      cos_sin_lambda += (max_factor<<2);
    }
}

/* 
   Accumulate the series given in instructions/coefficients.
   The argument of the series is cos_sin_lambda,
   initialized using PrepareElp82bLambdaArray().
   accu is the output accumulator.
   sp points to a memory area holding 2*nr_of_lambdas double.
   The area is overwritten  during the calculation.
*/
static void AccumulateElp82bTerms (const unsigned char *instructions,
				   const double *coefficients,
				   const double cos_sin_lambda[],
				   double accu[],
				   double *sp)
{
  sp[0] = 1.0;
  sp[1] = 0.0;
  
  for (;;)
    {
      int term_count = *instructions++;
      if (term_count < 0xFE)
	{
	  const int lambda_index = ((term_count&15)<<8)|(*instructions++);

	  /* Calculate new argument and push it on the stack */
	  const double *const cos_sin = cos_sin_lambda + (lambda_index<<1);
	  sp[2] = cos_sin[0]*sp[0] - cos_sin[1]*sp[1];
	  sp[3] = cos_sin[0]*sp[1] + cos_sin[1]*sp[0];
	  sp += 2;
	  term_count >>= 4;
	  while (--term_count >= 0)
	    {
	      accu[*instructions++] += (coefficients[0]*sp[0]
					+ coefficients[1]*sp[1]);
	      coefficients += 2;
	    }
	}
      else
	{
	  if (term_count == 0xFF) break;

	  /* Pop argument from the stack */
	  sp -= 2;
	}
    }
}

/* 
   Calculate spherical coordinates 
*/
static void GetElp82bSphericalCoord (const double t, double r[3])
{
  int    i, k;
  double lambda[17];
  double cos_sin_lambda[303*4];
  double accu[9];
  double stack[17*2];
  
  for (i = 0; i < 4; i++)
    {
      lambda[i] = 0.0;
      for (k = 3; k >= 0; k--)
	{
	  lambda[i] += del[k*4+i];
	  lambda[i] *= t;
	}
      lambda[5+i] = del[i]*t;
    }
  lambda[4] = zeta[0]*t;

  for (i = 0; i < 8; i++)
    {
      lambda[9+i] = p[i]*t;
    }

  PrepareElp82bLambdaArray (17, elp82b_max_lambda_factor, lambda, cos_sin_lambda);
  memcpy (accu, elp82b_constants, sizeof(elp82b_constants));
  AccumulateElp82bTerms (elp82b_instructions, elp82b_coefficients, cos_sin_lambda, accu, stack);

  /* Calculate r1, r2, r3: */
  r[0] =   (accu[0] + w[0]
       + t*(accu[3] + w[1]
       + t*(accu[6] + w[2]
       + t*(          w[3]
       + t*           w[4])))) * (M_PI/(180*3600));
  r[1] = (accu[1] + t*(accu[4] + t*accu[7])) * (M_PI/(180*3600));
  r[2] = (accu[2] + t*(accu[5] + t*accu[8])) * a0_div_ath_times_au;
}

/* 
   Calculate Cartesian coordinates 
*/
void GetElp82bCoord (const double jd, double xyz[3])
{
  const double t = (jd - 2451545.0) / 36525.0;
  double r[3];

  /* Get spherical coordinates */
  GetElp82bSphericalCoord (t, r);
  
  {
    /* Get Cartesian coordinates */
    const double rh = r[2] * cos(r[1]);
    const double x3 = r[2] * sin(r[1]);
    const double x1 = rh * cos(r[0]);
    const double x2 = rh * sin(r[0]);

    /* Construct elements of transformation matrix */
    double pw = t*(p1 + t*(p2 + t*(p3 + t*(p4 + t*p5))));
    double qw = t*(q1 + t*(q2 + t*(q3 + t*(q4 + t*q5))));
    const double pwq = pw * pw;
    const double qwq = qw * qw;
    const double pwqw = 2.0 * pw * qw;
    const double pw2 = 1.0 - 2.0 * pwq;
    const double qw2 = 1.0 - 2.0 * qwq;
    const double h = 1.0 - pwq - qwq;
    const double ra = (h > 0.0) ? (2.0 * sqrt(h)) : 0.0;
    pw *= ra;
    qw *= ra;

    /*
      Return geocentric Cartesian coordinates of Moon relative to
      inertial mean ecliptic and equinox of J2000 
    */
    xyz[0] = pw2 *x1 + pwqw*x2                + pw*x3;
    xyz[1] = pwqw*x1 + qw2 *x2                - qw*x3;
    xyz[2] = -pw *x1 + qw  *x2 + (pw2 + qw2 - 1.0)*x3;
  }
}


