#include "Eclipse.h"

// ==========================================================================
// Function to determine whether solar eclipse occurs at given new moon.
// t is Julian day number of first approximation to time of greatest eclipse.
// ==========================================================================
void TestEclipse (double t)
{
  // .................................
  // Get true time of greatest eclipse
  // .................................
  GetCentralTime (t);

  // ..........................................................
  // Test for total/annular eclipse at time of greatest eclipse
  // ...........................................................
  if (GetDelta (t) > 0.)
    {
      // Get Besselian elements at greatest eclipse
      double x, y, d, f1, f2, l1, l2, mu;
      GetBesselElements (t, x, y, d, f1, f2, l1, l2, mu);
      
      // Get derivatives of Besselian elements at greatest eclipse
      double xdot, ydot, mudot, ddot, l1dot, l2dot;
      GetBesselElementDerivatives (t, xdot, ydot, mudot, ddot, l1dot, l2dot);
      
      // Calculate gamma 
      double gamma = sqrt (x*x + y*y);
      if (y < 0)
	gamma = - gamma;
      
      // Calculate Dt at greatest eclipse
      double Dt = DeltaT (t);
      int nnn, sss;
      TimeConvert (Dt, nnn, sss);
      
      // Calculate TT date of greatest eclipse
      int YY, MM; double DD;
      JuliantoDate (t, YY, MM, DD);
      int dd, hh, nn, ss;
      DayNumberConvert (DD, dd, hh, nn, ss);

      // Calculate UT date of greatest eclipse
      double JDU = t - DeltaT (t) /day;
      int YYU, MMU; double DDU;
      JuliantoDate (JDU, YYU, MMU, DDU);
      int ddU, hhU, nnU, ssU;
      DayNumberConvert (DDU, ddU, hhU, nnU, ssU);
      
      // Get equatorial coordinates of Sun at greatest eclipse 
      double Sun_r, Sun_delta, Sun_RA;
      SunEquatorial (t, Sun_r, Sun_delta, Sun_RA, 1, 1, 1, 1);
      int SSb1, SSb2, SSb3, SSl1, SSl2, SSl3;
      DeclinationConvert    (Sun_delta, SSb1, SSb2, SSb3);
      RightAscensionConvert (Sun_RA,    SSl1, SSl2, SSl3);

      // Get equatorial coordinates of Moon at greatest eclipse 
      double Moon_r, Moon_delta, Moon_RA;
      MoonEquatorial (t, Moon_r, Moon_delta, Moon_RA, 1, 1, 1, 1);
      int MMb1, MMb2, MMb3, MMl1, MMl2, MMl3;
      DeclinationConvert    (Moon_delta, MMb1, MMb2, MMb3);
      RightAscensionConvert (Moon_RA,    MMl1, MMl2, MMl3);

      // Get hour angle, latitude, and longitude of greatest eclipse
      double B, zeta, u, phi, H, lambda;
      GetLatituteLongitude (x, y, d, mu, B, zeta, u, phi, H, lambda);
      int LL1, LL2, LL3, PP1, PP2, PP3;
      DeclinationConvert (phi, PP1, PP2, PP3);
      DeclinationConvert (lambda, LL1, LL2, LL3);

      // Get solar altitude at greatest eclipse
      double Sun_Alt;
      GetSolarAltitude (t, phi, lambda, Sun_Alt);
      int AA1, AA2, AA3;
      DeclinationConvert (Sun_Alt, AA1, AA2, AA3);

      // Get eclipse magnitude at greatest eclipse
      double Mg, L1, L2;
      GetL1L2 (f1, f2, l1, l2, zeta, L1, L2);
      GetCentralEclipseMagnitude (L1, L2, Mg);
  
      // Get eclipse duration at greatest eclipse
      double tau;
      GetCentralEclipseDuration (x, y, d, f2, xdot, ydot, mudot, ddot, l2dot, zeta, L2, tau);
      int MMd, SSd;
      TimeConvert (tau, MMd, SSd);

      // Get umbral external/internal contact times
      double U1 = t, U2 = t, U3 = t, U4 = t;
      GetUP1 (U1, 0);
      GetUP2 (U2, 0);
      GetUP3 (U3, 0);
      GetUP4 (U4, 0);
      int YYU1, MMU1; double DDU1;
      JuliantoDate (U1 - DeltaT (t) /day, YYU1, MMU1, DDU1);
      int ddU1, hhU1, nnU1, ssU1;
      DayNumberConvert (DDU1, ddU1, hhU1, nnU1, ssU1);
      int YYU4, MMU4; double DDU4;
      JuliantoDate (U4 - DeltaT (t) /day, YYU4, MMU4, DDU4);
      int ddU4, hhU4, nnU4, ssU4;
      DayNumberConvert (DDU4, ddU4, hhU4, nnU4, ssU4);
      int YYU2, MMU2; double DDU2;
      int ddU2, hhU2, nnU2, ssU2;
      int YYU3, MMU3; double DDU3;
      int ddU3, hhU3, nnU3, ssU3;
      if (U2 > -1.e14)
	{
	  JuliantoDate (U2 - DeltaT (t) /day, YYU2, MMU2, DDU2);
	  DayNumberConvert (DDU2, ddU2, hhU2, nnU2, ssU2);
	  JuliantoDate (U3 - DeltaT (t) /day, YYU3, MMU3, DDU3);
	  DayNumberConvert (DDU3, ddU3, hhU3, nnU3, ssU3);
  	}

      // Get penumbral external/internal contact times
      double P1 = t, P2 = t, P3 = t, P4 = t;
      GetUP1 (P1, 1);
      GetUP2 (P2, 1);
      GetUP3 (P3, 1);
      GetUP4 (P4, 1);
      int YYP1, MMP1; double DDP1;
      JuliantoDate (P1 - DeltaT (t) /day, YYP1, MMP1, DDP1);
      int ddP1, hhP1, nnP1, ssP1;
      DayNumberConvert (DDP1, ddP1, hhP1, nnP1, ssP1);
      int YYP4, MMP4; double DDP4;
      JuliantoDate (P4 - DeltaT (t) /day, YYP4, MMP4, DDP4);
      int ddP4, hhP4, nnP4, ssP4;
      DayNumberConvert (DDP4, ddP4, hhP4, nnP4, ssP4);
      int YYP2, MMP2; double DDP2;
      int ddP2, hhP2, nnP2, ssP2;
      int YYP3, MMP3; double DDP3;
      int ddP3, hhP3, nnP3, ssP3;
      if (P2 > -1.e14)
	{
	  JuliantoDate (P2 - DeltaT (t) /day, YYP2, MMP2, DDP2);
	  DayNumberConvert (DDP2, ddP2, hhP2, nnP2, ssP2);
	  JuliantoDate (P3 - DeltaT (t) /day, YYP3, MMP3, DDP3);
	  DayNumberConvert (DDP3, ddP3, hhP3, nnP3, ssP3);
  	}

      // Construct filenames
      const char *MO[12];
      MO[0] = "Jan"; MO[1] = "Feb"; MO[ 2] = "Mar"; MO[ 3] = "Apr";
      MO[4] = "May"; MO[5] = "Jun"; MO[ 6] = "Jul"; MO[ 7] = "Aug";
      MO[8] = "Sep"; MO[9] = "Oct"; MO[10] = "Nov"; MO[11] = "Dec";
      char GeneralFile           [60]; char BesselFile            [60]; char GreatestFile [60];
      char TrackCentralFile      [60]; char UmbralShadowFile      [60]; char P1234File    [60];
      char PenumbralContact1File [60]; char PenumbralContact2File [60];
      char Magnitude00File       [60]; char Magnitude01File       [60];
      char Magnitude02File       [60]; char Magnitude03File       [60];
      char Magnitude04File       [60]; char Magnitude05File       [60];
      char Magnitude06File       [60]; char Magnitude07File       [60];
      char Magnitude08File       [60]; char Magnitude09File       [60];
      char Magnitude10File       [60]; char Magnitude11File       [60];
      char Magnitude12File       [60]; char Magnitude13File       [60];
      char Magnitude14File       [60]; char Magnitude15File       [60];
      char Magnitude16File       [60]; char Magnitude17File       [60];
      char Magnitude18File       [60]; char Magnitude19File       [60];
      char Magnitude20File       [60]; char Magnitude21File       [60];
      char Magnitude22File       [60]; char Magnitude23File       [60];
      char Maximum00File         [60]; char Maximum01File         [60];
      char Maximum02File         [60]; char Maximum03File         [60];
      char Maximum04File         [60]; char Maximum05File         [60];
      char Maximum06File         [60]; char Maximum07File         [60];
      char Maximum08File         [60]; char Maximum09File         [60];
      char Maximum10File         [60]; char Maximum11File         [60];
      char Maximum12File         [60]; char Maximum13File         [60];
      char Maximum14File         [60]; char Maximum15File         [60];
      char Maximum16File         [60]; char Maximum17File         [60];
      ConstructFilenames (MM, MO, GeneralFile, BesselFile, GreatestFile, TrackCentralFile, UmbralShadowFile,
			  P1234File,      PenumbralContact1File, PenumbralContact2File,
			  Magnitude00File, Magnitude01File, Magnitude02File, Magnitude03File,
			  Magnitude04File, Magnitude05File, Magnitude06File, Magnitude07File,
			  Magnitude08File, Magnitude09File, Magnitude10File, Magnitude11File,
			  Magnitude12File, Magnitude13File, Magnitude14File, Magnitude15File,
			  Magnitude16File, Magnitude17File, Magnitude18File, Magnitude19File,
			  Magnitude20File, Magnitude21File, Magnitude22File, Magnitude23File,
			  Maximum00File,   Maximum01File,   Maximum02File,   Maximum03File,
			  Maximum04File,   Maximum05File,   Maximum06File,   Maximum07File,
			  Maximum08File,   Maximum09File,   Maximum10File,   Maximum11File,
			  Maximum12File,   Maximum13File,   Maximum14File,   Maximum15File,
			  Maximum16File,   Maximum17File);
	   
      // Output general eclipse data
      printf ("Greatest Eclipse:\n\n");
      printf ("%4d %3s %02d, %02d:%02d:%02d TDT :: %02d:%02d:%02d UT :: DeltaT = %02d:%02d :: gamma = %+8.5f :: Magnitude = %7.5f :: Duration = %02d:%02d\n",
	      YY, MO[MM-1], dd, hh, nn, ss, hhU, nnU, ssU, nnn, sss, gamma, Mg, MMd, SSd);
      printf ("\nSun R.A.: %03d:%02d:%02d :: Sun Dec.: %+03d:%02d:%02d :: Moon R.A.: %03d:%02d:%02d :: Moon Dec.: %+03d:%02d:%02d\n",
	      SSl1, SSl2, SSl3, SSb1, SSb2, SSb3, MMl1, MMl2, MMl3, MMb1, MMb2, MMb3);
      printf ("\nLongitude: %+04d:%02d:%02d :: Latitude: %+03d:%02d:%02d :: Solar Altitude:  %+02d:%02d:%02d\n",
	      LL1, LL2, LL3, PP1, PP2, PP3, AA1, AA2, AA3);

      // Output external/internal contact times of umbra and penumbra
      printf ("\nExternal/Internal Contact Times of Umbra and Penumbra:\n\n");
      if (U2 > -1.e14)
	printf ("U1 = %02d:%02d:%02d UT :: U2 = %02d:%02d:%02d UT :: U3 = %02d:%02d:%02d UT :: U4 = %02d:%02d:%02d UT\n",
		hhU1, nnU1, ssU1, hhU2, nnU2, ssU2, hhU3, nnU3, ssU3, hhU4, nnU4, ssU4);
      else
	printf ("U1 = %02d:%02d:%02d UT :: U4 =  %02d:%02d:%02d UT\n",
		hhU1, nnU1, ssU1, hhU4, nnU4, ssU4);
      if (P2 > -1.e14)
	printf ("P1 = %02d:%02d:%02d UT :: P2 = %02d:%02d:%02d UT :: P3 = %02d:%02d:%02d UT :: P4 = %02d:%02d:%02d UT\n",
		hhP1, nnP1, ssP1, hhP2, nnP2, ssP2, hhP3, nnP3, ssP3, hhP4, nnP4, ssP4);
      else
	printf ("P1 = %02d:%02d:%02d UT :: P4 = %02d:%02d:%02d UT\n",
		hhP1, nnP1, ssP1, hhP4, nnP4, ssP4);
 	  
      FILE *file = OpenFile (GeneralFile, "w");
      fprintf (file, "Greatest Eclipse:\n\n");
      fprintf (file, "%4d %3s %2d, %02d:%02d:%02d TDT :: %02d:%02d:%02d UT :: DeltaT = %02d:%02d :: gamma = %+8.5f :: Magnitude = %7.5f :: Duration = %02d:%02d\n",
	       YY, MO[MM-1], dd, hh, nn, ss, hhU, nnU, ssU, nnn, sss, gamma, Mg, MMd, SSd);
      fprintf (file, "\nSun R.A.: %03d:%02d:%02d :: Sun Dec.: %+03d:%02d:%02d :: Moon R.A.: %03d:%02d:%02d :: Moon Dec.: %+03d:%02d:%02d\n",
	       SSl1, SSl2, SSl3, SSb1, SSb2, SSb3, MMl1, MMl2, MMl3, MMb1, MMb2, MMb3);
      fprintf (file, "\nLongitude: %+04d:%02d:%02d :: Latitude: %+03d:%02d:%02d :: Solar Altitude:  %+02d:%02d:%02d\n",
	       LL1, LL2, LL3, PP1, PP2, PP3, AA1, AA2, AA3);
      fprintf (file, "\nExternal/Internal Contact Times of Umbra and Penumbra:\n\n");
      if (U2 > -1.e14)
	fprintf (file, "U1 = %02d:%02d:%02d UT :: U2 = %02d:%02d:%02d UT :: U3 = %02d:%02d:%02d UT :: U4 = %02d:%02d:%02d UT\n",
		hhU1, nnU1, ssU1, hhU2, nnU2, ssU2, hhU3, nnU3, ssU3, hhU4, nnU4, ssU4);
      else
	fprintf (file, "U1 = %02d:%02d:%02d UT :: U4 = %02d:%02d:%02d UT\n",
		hhU1, nnU1, ssU1, hhU4, nnU4, ssU4);
      if (P2 > -1.e14)
	fprintf (file, "P1 = %02d:%02d:%02d UT :: P2 = %02d:%02d:%02d UT :: P3 = %02d:%02d:%02d UT :: P4 = %02d:%02d:%02d UT\n",
		hhP1, nnP1, ssP1, hhP2, nnP2, ssP2, hhP3, nnP3, ssP3, hhP4, nnP4, ssP4);
      else
	fprintf (file, "P1 = %02d:%02d:%02d UT :: P4 = %02d:%02d:%02d UT\n",
		hhP1, nnP1, ssP1, hhP4, nnP4, ssP4);
      fclose (file);

      // Output table of Besselian elements
      WriteBesselElementsTable (YY, MM, dd, hh, tan (f1), tan (f2), mudot, BesselFile);

      // Track central line of eclipse
      printf ("Tracking Central Line:\n");
      TrackCentralLine (t, GreatestFile, TrackCentralFile, UmbralShadowFile);

      // Track penumbral contact curve
      printf ("Tracking Penumbral Contact Curve:\n");
      TrackPenumbralContact (t, P1234File, PenumbralContact1File, PenumbralContact2File);

      // Track points of given eclipse magnitude at maximum eclipse
      printf ("Tracking Points of Constant Eclipse Magnitude at Maximum Eclipse:\n");
      TrackMagnitude (t, P1, P4,
		      Magnitude00File, Magnitude01File, Magnitude02File, Magnitude03File,
      		      Magnitude04File, Magnitude05File, Magnitude06File, Magnitude07File,
      		      Magnitude08File, Magnitude09File, Magnitude10File, Magnitude11File,
		      Magnitude12File, Magnitude13File, Magnitude14File, Magnitude15File,
      		      Magnitude16File, Magnitude17File, Magnitude18File, Magnitude19File,
      		      Magnitude20File, Magnitude21File, Magnitude22File, Magnitude23File);
      
      // Get reduced Julian day number of central integer half hour
      double EEU, t0;
      DayNumberReduce (DDU, EEU);
      DatetoJulian (YYU, MMU, EEU, t0);
      DayNumberConvert (EEU, ddU, hhU, nnU, ssU);

      // Track points of maximum eclipse at fixed half-hour time intervals
      printf ("Tracking Points of Maximum Eclipse at Constant Time: tc = %2d:%02d:%02d UT\n", hhU, nnU, ssU);
      TrackMaximum (t0, P1, P4, Mg, 
		    Maximum00File, Maximum01File, Maximum02File, Maximum03File,
		    Maximum04File, Maximum05File, Maximum06File, Maximum07File,
		    Maximum08File, Maximum09File, Maximum10File, Maximum11File,
		    Maximum12File, Maximum13File, Maximum14File, Maximum15File,
		    Maximum16File, Maximum17File);
      printf ("\n");
    }

  // ....................................................
  // Test for partial eclipse at time of greatest eclipse
  // ....................................................
  else if (GetDelta_max (t, 1) > 0.)
    {
      // Get Besselian elements at greatest eclipse
      double x, y, d, f1, f2, l1, l2, mu;
      GetBesselElements (t, x, y, d, f1, f2, l1, l2, mu);
      
      // Get derivatives of Besselian elements at greatest eclipse
      double xdot, ydot, mudot, ddot, l1dot, l2dot;
      GetBesselElementDerivatives (t, xdot, ydot, mudot, ddot, l1dot, l2dot);
      
      // Calculate gamma 
      double gamma = sqrt (x*x + y*y);
      if (y < 0)
	gamma = - gamma;
      
      // Calculate Dt at greatest eclipse
      double Dt = DeltaT (t);
      int nnn, sss;
      TimeConvert (Dt, nnn, sss);
      
      // Calculate TT date of greatest eclipse
      int YY, MM; double DD;
      JuliantoDate (t, YY, MM, DD);
      int dd, hh, nn, ss;
      DayNumberConvert (DD, dd, hh, nn, ss);

      // Calculate UT date of greatest eclipse
      double JDU = t - DeltaT (t) /day;
      int YYU, MMU; double DDU;
      JuliantoDate (JDU, YYU, MMU, DDU);
      int ddU, hhU, nnU, ssU;
      DayNumberConvert (DDU, ddU, hhU, nnU, ssU);
      
      // Get equatorial coordinates of Sun at greatest eclipse 
      double Sun_r, Sun_delta, Sun_RA;
      SunEquatorial (t, Sun_r, Sun_delta, Sun_RA, 1, 1, 1, 1);
      int SSb1, SSb2, SSb3, SSl1, SSl2, SSl3;
      DeclinationConvert    (Sun_delta, SSb1, SSb2, SSb3);
      RightAscensionConvert (Sun_RA,    SSl1, SSl2, SSl3);

      // Get equatorial coordinates of Moon at greatest eclipse 
      double Moon_r, Moon_delta, Moon_RA;
      MoonEquatorial (t, Moon_r, Moon_delta, Moon_RA, 1, 1, 1, 1);
      int MMb1, MMb2, MMb3, MMl1, MMl2, MMl3;
      DeclinationConvert    (Moon_delta, MMb1, MMb2, MMb3);
      RightAscensionConvert (Moon_RA,    MMl1, MMl2, MMl3);

      // Get hour angle, latitude, and longitude of greatest eclipse
      double B, zeta, u, phi, H, lambda;
      double omega =  1. /sqrt (1. - fe * (2. - fe) * cos (d) * cos (d));
      double fac   = sqrt (x*x + omega*omega * y*y);
      GetLatituteLongitude (x/fac, y/fac, d, mu, B, zeta, u, phi, H, lambda);

      // Get eclipse magnitude at greatest eclipse
      double Mg, L1, L2;
      GetL1L2 (f1, f2, l1, l2, zeta, L1, L2);
      GetEclipseMagnitude (x, y, x/fac, y/fac, L1, L2, Mg);
 
      // Get penumbral external/internal contact times
      double P1 = t, P2 = t, P3 = t, P4 = t;
      GetUP1 (P1, 1);
      GetUP2 (P2, 1);
      GetUP3 (P3, 1);
      GetUP4 (P4, 1);
      int YYP1, MMP1; double DDP1;
      JuliantoDate (P1 - DeltaT (t) /day, YYP1, MMP1, DDP1);
      int ddP1, hhP1, nnP1, ssP1;
      DayNumberConvert (DDP1, ddP1, hhP1, nnP1, ssP1);
      int YYP4, MMP4; double DDP4;
      JuliantoDate (P4 - DeltaT (t) /day, YYP4, MMP4, DDP4);
      int ddP4, hhP4, nnP4, ssP4;
      DayNumberConvert (DDP4, ddP4, hhP4, nnP4, ssP4);
      int YYP2, MMP2; double DDP2;
      int ddP2, hhP2, nnP2, ssP2;
      int YYP3, MMP3; double DDP3;
      int ddP3, hhP3, nnP3, ssP3;
      if (P2 > -1.e14)
	{
	  JuliantoDate (P2 - DeltaT (t) /day, YYP2, MMP2, DDP2);
	  DayNumberConvert (DDP2, ddP2, hhP2, nnP2, ssP2);
	  JuliantoDate (P3 - DeltaT (t) /day, YYP3, MMP3, DDP3);
	  DayNumberConvert (DDP3, ddP3, hhP3, nnP3, ssP3);
  	}

      // Construct filenames
      const char *MO[12];
      MO[0] = "Jan"; MO[1] = "Feb"; MO[ 2] = "Mar"; MO[ 3] = "Apr";
      MO[4] = "May"; MO[5] = "Jun"; MO[ 6] = "Jul"; MO[ 7] = "Aug";
      MO[8] = "Sep"; MO[9] = "Oct"; MO[10] = "Nov"; MO[11] = "Dec";
      char GeneralFile           [60]; char BesselFile            [60]; char GreatestFile [60];
      char TrackCentralFile      [60]; char UmbralShadowFile      [60]; char P1234File    [60];
      char PenumbralContact1File [60]; char PenumbralContact2File [60];
      char Magnitude00File       [60]; char Magnitude01File       [60];
      char Magnitude02File       [60]; char Magnitude03File       [60];
      char Magnitude04File       [60]; char Magnitude05File       [60];
      char Magnitude06File       [60]; char Magnitude07File       [60];
      char Magnitude08File       [60]; char Magnitude09File       [60];
      char Magnitude10File       [60]; char Magnitude11File       [60];
      char Magnitude12File       [60]; char Magnitude13File       [60];
      char Magnitude14File       [60]; char Magnitude15File       [60];
      char Magnitude16File       [60]; char Magnitude17File       [60];
      char Magnitude18File       [60]; char Magnitude19File       [60];
      char Magnitude20File       [60]; char Magnitude21File       [60];
      char Magnitude22File       [60]; char Magnitude23File       [60];
      char Maximum00File         [60]; char Maximum01File         [60];
      char Maximum02File         [60]; char Maximum03File         [60];
      char Maximum04File         [60]; char Maximum05File         [60];
      char Maximum06File         [60]; char Maximum07File         [60];
      char Maximum08File         [60]; char Maximum09File         [60];
      char Maximum10File         [60]; char Maximum11File         [60];
      char Maximum12File         [60]; char Maximum13File         [60];
      char Maximum14File         [60]; char Maximum15File         [60];
      char Maximum16File         [60]; char Maximum17File         [60];
      ConstructFilenames (MM, MO, GeneralFile, BesselFile, GreatestFile, TrackCentralFile, UmbralShadowFile,
			  P1234File,      PenumbralContact1File, PenumbralContact2File,
			  Magnitude00File, Magnitude01File, Magnitude02File, Magnitude03File,
			  Magnitude04File, Magnitude05File, Magnitude06File, Magnitude07File,
			  Magnitude08File, Magnitude09File, Magnitude10File, Magnitude11File,
			  Magnitude12File, Magnitude13File, Magnitude14File, Magnitude15File,
			  Magnitude16File, Magnitude17File, Magnitude18File, Magnitude19File,
			  Magnitude20File, Magnitude21File, Magnitude22File, Magnitude23File,
			  Maximum00File,   Maximum01File,   Maximum02File,   Maximum03File,
			  Maximum04File,   Maximum05File,   Maximum06File,   Maximum07File,
			  Maximum08File,   Maximum09File,   Maximum10File,   Maximum11File,
			  Maximum12File,   Maximum13File,   Maximum14File,   Maximum15File,
			  Maximum16File,   Maximum17File);
	   
      // Output general eclipse data
      printf ("Greatest Eclipse:\n\n");
      printf ("%4d %3s %02d, %02d:%02d:%02d TDT :: %02d:%02d:%02d UT :: DeltaT = %02d:%02d :: gamma = %+8.5f :: Magnitude = %7.5f\n",
	      YY, MO[MM-1], dd, hh, nn, ss, hhU, nnU, ssU, nnn, sss, gamma, Mg);
      printf ("\nSun R.A.: %03d:%02d:%02d :: Sun Dec.: %+03d:%02d:%02d :: Moon R.A.: %03d:%02d:%02d :: Moon Dec.: %+03d:%02d:%02d\n",
	      SSl1, SSl2, SSl3, SSb1, SSb2, SSb3, MMl1, MMl2, MMl3, MMb1, MMb2, MMb3);

      // Output external/internal contact times of penumbra 
      printf ("\nExternal/Internal Contact Times of Penumbra:\n\n");
      if (P2 > -1.e14)
	printf ("P1 = %02d:%02d:%02d UT :: P2 = %02d:%02d:%02d UT :: P3 = %02d:%02d:%02d UT :: P4 = %02d:%02d:%02d UT\n",
		hhP1, nnP1, ssP1, hhP2, nnP2, ssP2, hhP3, nnP3, ssP3, hhP4, nnP4, ssP4);
      else
	printf ("P1 = %02d:%02d:%02d UT :: P4 = %02d:%02d:%02d UT\n",
		hhP1, nnP1, ssP1, hhP4, nnP4, ssP4);
 	  
      FILE *file = OpenFile (GeneralFile, "w");
      fprintf (file, "Greatest Eclipse:\n\n");
      fprintf (file, "%4d %3s %2d, %02d:%02d:%02d TDT :: %02d:%02d:%02d UT :: DeltaT = %02d:%02d :: gamma = %+8.5f :: Magnitude = %7.5f\n",
	       YY, MO[MM-1], dd, hh, nn, ss, hhU, nnU, ssU, nnn, sss, gamma, Mg);
      fprintf (file, "\nSun R.A.: %03d:%02d:%02d :: Sun Dec.: %+03d:%02d:%02d :: Moon R.A.: %03d:%02d:%02d :: Moon Dec.: %+03d:%02d:%02d\n",
	       SSl1, SSl2, SSl3, SSb1, SSb2, SSb3, MMl1, MMl2, MMl3, MMb1, MMb2, MMb3);
      fprintf (file, "\nExternal/Internal Contact Times of Penumbra:\n\n");
      if (P2 > -1.e14)
	fprintf (file, "P1 = %02d:%02d:%02d UT :: P2 = %02d:%02d:%02d UT :: P3 = %02d:%02d:%02d UT :: P4 = %02d:%02d:%02d UT\n",
		hhP1, nnP1, ssP1, hhP2, nnP2, ssP2, hhP3, nnP3, ssP3, hhP4, nnP4, ssP4);
      else
	fprintf (file, "P1 = %02d:%02d:%02d UT :: P4 = %02d:%02d:%02d UT\n",
		hhP1, nnP1, ssP1, hhP4, nnP4, ssP4);
      fclose (file);

      // Output table of Besselian elements
      WriteBesselElementsTable (YY, MM, dd, hh, tan (f1), tan (f2), mudot, BesselFile);

      // Track central line of eclipse
      TrackCentralLine1 (t, P1, P4, GreatestFile, TrackCentralFile, UmbralShadowFile);

      // Track penumbral contact curve
      printf ("Tracking Penumbral Contact Curve:\n");
      TrackPenumbralContact (t, P1234File, PenumbralContact1File, PenumbralContact2File);

      // Track points of given eclipse magnitude at maximum eclipse
      printf ("Tracking Points of Constant Eclipse Magnitude at Maximum Eclipse:\n");
      TrackMagnitude (t, P1, P4,
		      Magnitude00File, Magnitude01File, Magnitude02File, Magnitude03File,
      		      Magnitude04File, Magnitude05File, Magnitude06File, Magnitude07File,
      		      Magnitude08File, Magnitude09File, Magnitude10File, Magnitude11File,
		      Magnitude12File, Magnitude13File, Magnitude14File, Magnitude15File,
      		      Magnitude16File, Magnitude17File, Magnitude18File, Magnitude19File,
      		      Magnitude20File, Magnitude21File, Magnitude22File, Magnitude23File);
      
      // Get reduced Julian day number of central integer half hour
      double EEU, t0;
      DayNumberReduce (DDU, EEU);
      DatetoJulian (YYU, MMU, EEU, t0);
      DayNumberConvert (EEU, ddU, hhU, nnU, ssU);

      // Track points of maximum eclipse at fixed half-hour time intervals
      printf ("Tracking Points of Maximum Eclipse at Constant Time: tc = %2d:%02d:%02d UT\n", hhU, nnU, ssU);
      TrackMaximum (t0, P1, P4, Mg, 
		    Maximum00File, Maximum01File, Maximum02File, Maximum03File,
		    Maximum04File, Maximum05File, Maximum06File, Maximum07File,
		    Maximum08File, Maximum09File, Maximum10File, Maximum11File,
		    Maximum12File, Maximum13File, Maximum14File, Maximum15File,
		    Maximum16File, Maximum17File);
      printf ("\n");
    }
}

// =======================
// Function to return zeta
// =======================
void GetZeta (double xi, double eta, double d, double &B, double& zeta)
{
  double omega = 1. /sqrt (1. - fe * (2. - fe) * cos (d) * cos (d));
  double eta1  = omega * eta;

  double D = 1. - xi*xi - eta1*eta1;
  if (D > 0.)
    B = sqrt (D);
  else
    B = 0.;

  zeta = - omega * fe * (2. - fe) * cos (d) * sin (d) * eta1 + omega * (1. - fe) * B;
}

// =======================================
// Function to return zeta on Earth's limb
// =======================================
void GetZeta0 (double xi, double eta, double d, double omega, double& zeta)
{
  double eta1  = omega * eta;

  zeta = - omega * fe * (2. - fe) * cos (d) * sin (d) * eta1;
}

// ======================================================
// Function to return hour angle, latitude, and longitude
// ======================================================
void GetLatituteLongitude (double xi, double eta, double d, double mu, double &B, double& zeta,
			   double &u, double &phi, double &H, double &lambda)
{
  double omega = 1. /sqrt (1. - fe * (2. - fe) * cos (d) * cos (d));
  double eta1  = omega * eta;
  double b1    = omega * sin (d);
  double b2    = (1. - fe) * omega * cos (d);

  double D = 1. - xi*xi - eta1*eta1;
  if (D > 0.)
    B = sqrt (D);
  else
    B = 0.;

  zeta = - omega * fe * (2. - fe) * cos (d) * sin (d) * eta1 + omega * (1. - fe) * B;

  double fac = b1 * B + b2 * eta1; 
  if (fac > 1.)
    {
      u   = M_PI/2.;
      phi = M_PI/2;
    }
  else if (fac < -1.)
    {
      u   = - M_PI/2.;
      phi = - M_PI/2.;
    }
  else
    {
      u   = asin (fac);
      phi = atan (tan (u) /(1. - fe));
    }

  H      = atan2 (xi, b2 * B - b1 * eta1);
  lambda = mu - H;
  lambda = atan2 (sin (lambda), cos (lambda));
}

// ============================
// Function to return L1 and L2
// ============================
void GetL1L2 (double f1, double f2, double l1, double l2, double zeta, double &L1, double &L2)
{
  L1  = l1 - zeta * tan (f1);
  L2  = l2 - zeta * tan (f2);
}

// ============================================================================
// Function to return solar altitude at given time and place on Earth's surface
// ============================================================================
void GetSolarAltitude (double t, double phi, double lambda, double &Sun_Alt)
{
  // Get equatorial coordinates of Sun
  double r, delta, RA;
  SunEquatorial (t, r, delta, RA, 1, 1, 1, 1);

  // Calculate mean Greenwich sidereal time (in radians)
  double theta = MeanGreenwichSiderealTime (t);

  // Calculate hour angle of Sun
  double H = theta - RA - lambda;

  // Calculate altitude
  double sinalt = sin (delta) * sin (phi) + cos (delta) * cos (phi) * cos (H);

  Sun_Alt = asin (sinalt);
}

// ========================================================================
// Function to return eclipse magnitude at point on central line of eclipse
// ========================================================================
void GetEclipseMagnitude (double x, double y, double xi, double eta, double L1, double L2, double &Mg)
{
  double m   = sqrt ((xi - x) * (xi - x) + (eta - y) * (eta - y));
  
  Mg  = (L1 - m) /(L1 + L2);
}

// ============================================================================
// Function to return eclipse magnitude at point not on central line of eclipse
// ============================================================================
void GetCentralEclipseMagnitude (double L1, double L2, double &Mg)
{
  Mg  = (L1 - L2) /(L1 + L2);
}

// =======================================================================
// Function to return eclipse duration at point on central line of eclipse
// =======================================================================
void GetCentralEclipseDuration (double x, double y, double d, double f2, double xdot, double ydot, double mudot, double ddot,
				double l2dot, double zeta, double L2, double &tau)
{
  double a    = xdot + mudot * (y * sin (d) - zeta * cos (d));
  double b    = ydot - mudot * x * sin (d) + ddot * zeta;
  double N    = sqrt (a*a + b*b);
  double c    = l2dot + (mudot * x * cos (d) - ddot * y) * tan (f2);

  tau = 2. * day * fabs (L2) * N /(N*N - c*c);
}

// ...............................
// Function to construct filenames
// ...............................
void ConstructFilenames (int MM, const char *MO[], char GeneralFile[],      char BesselFile[], char GreatestFile[],
			 char TrackCentralFile[],  char UmbralShadowFile[], char P1234File[],
			 char PenumbralContact1File[], char PenumbralContact2File[],
			 char Magnitude00File[], char Magnitude01File[],
			 char Magnitude02File[], char Magnitude03File[],
			 char Magnitude04File[], char Magnitude05File[],
			 char Magnitude06File[], char Magnitude07File[],
			 char Magnitude08File[], char Magnitude09File[],
			 char Magnitude10File[], char Magnitude11File[],
			 char Magnitude12File[], char Magnitude13File[],
			 char Magnitude14File[], char Magnitude15File[],
			 char Magnitude16File[], char Magnitude17File[],
			 char Magnitude18File[], char Magnitude19File[],
			 char Magnitude20File[], char Magnitude21File[],
			 char Magnitude22File[], char Magnitude23File[],
			 char Maximum00File[],   char Maximum01File[],
			 char Maximum02File[],   char Maximum03File[],
			 char Maximum04File[],   char Maximum05File[],
			 char Maximum06File[],   char Maximum07File[],
			 char Maximum08File[],   char Maximum09File[],
			 char Maximum10File[],   char Maximum11File[],
			 char Maximum12File[],   char Maximum13File[],
			 char Maximum14File[],   char Maximum15File[],
			 char Maximum16File[],   char Maximum17File[])
{
  strcpy (GeneralFile,           "Output/");
  strcpy (BesselFile,            "Output/");
  strcpy (GreatestFile,          "Output/");
  strcpy (TrackCentralFile,      "Output/");
  strcpy (UmbralShadowFile,      "Output/");
  strcpy (P1234File,             "Output/");
  strcpy (PenumbralContact1File, "Output/");
  strcpy (PenumbralContact2File, "Output/");
  strcpy (Magnitude00File,       "Output/");
  strcpy (Magnitude01File,       "Output/");
  strcpy (Magnitude02File,       "Output/");
  strcpy (Magnitude03File,       "Output/");
  strcpy (Magnitude04File,       "Output/");
  strcpy (Magnitude05File,       "Output/");
  strcpy (Magnitude06File,       "Output/");
  strcpy (Magnitude07File,       "Output/");
  strcpy (Magnitude08File,       "Output/");
  strcpy (Magnitude09File,       "Output/");
  strcpy (Magnitude10File,       "Output/");
  strcpy (Magnitude11File,       "Output/");
  strcpy (Magnitude12File,       "Output/");
  strcpy (Magnitude13File,       "Output/");
  strcpy (Magnitude14File,       "Output/");
  strcpy (Magnitude15File,       "Output/");
  strcpy (Magnitude16File,       "Output/");
  strcpy (Magnitude17File,       "Output/");
  strcpy (Magnitude18File,       "Output/");
  strcpy (Magnitude19File,       "Output/");
  strcpy (Magnitude20File,       "Output/");
  strcpy (Magnitude21File,       "Output/");
  strcpy (Magnitude22File,       "Output/");
  strcpy (Magnitude23File,       "Output/");
  strcpy (Maximum00File,         "Output/");
  strcpy (Maximum01File,         "Output/");
  strcpy (Maximum02File,         "Output/");
  strcpy (Maximum03File,         "Output/");
  strcpy (Maximum04File,         "Output/");
  strcpy (Maximum05File,         "Output/");
  strcpy (Maximum06File,         "Output/");
  strcpy (Maximum07File,         "Output/");
  strcpy (Maximum08File,         "Output/");
  strcpy (Maximum09File,         "Output/");
  strcpy (Maximum10File,         "Output/");
  strcpy (Maximum11File,         "Output/");
  strcpy (Maximum12File,         "Output/");
  strcpy (Maximum13File,         "Output/");
  strcpy (Maximum14File,         "Output/");
  strcpy (Maximum15File,         "Output/");
  strcpy (Maximum16File,         "Output/");
  strcpy (Maximum17File,         "Output/");
  
  char General           [30] = "/General.out";
  char Bessel            [30] = "/Bessel.out";
  char Greatest          [30] = "/Greatest.out";
  char TrackCentral      [30] = "/TrackCentral.out";
  char UmbralShadow      [30] = "/UmbralShadow.out";
  char P1234             [30] = "/P1234.out";
  char PenumbralContact1 [30] = "/PenumbralContact/1.out";
  char PenumbralContact2 [30] = "/PenumbralContact/2.out";
  char Magnitude00       [30] = "/Magnitude/00.out";
  char Magnitude01       [30] = "/Magnitude/01.out";
  char Magnitude02       [30] = "/Magnitude/02.out";
  char Magnitude03       [30] = "/Magnitude/03.out";
  char Magnitude04       [30] = "/Magnitude/04.out";
  char Magnitude05       [30] = "/Magnitude/05.out";
  char Magnitude06       [30] = "/Magnitude/06.out";
  char Magnitude07       [30] = "/Magnitude/07.out";
  char Magnitude08       [30] = "/Magnitude/08.out";
  char Magnitude09       [30] = "/Magnitude/09.out";
  char Magnitude10       [30] = "/Magnitude/10.out";
  char Magnitude11       [30] = "/Magnitude/11.out";
  char Magnitude12       [30] = "/Magnitude/12.out";
  char Magnitude13       [30] = "/Magnitude/13.out";
  char Magnitude14       [30] = "/Magnitude/14.out";
  char Magnitude15       [30] = "/Magnitude/15.out";
  char Magnitude16       [30] = "/Magnitude/16.out";
  char Magnitude17       [30] = "/Magnitude/17.out";
  char Magnitude18       [30] = "/Magnitude/18.out";
  char Magnitude19       [30] = "/Magnitude/19.out";
  char Magnitude20       [30] = "/Magnitude/20.out";
  char Magnitude21       [30] = "/Magnitude/21.out";
  char Magnitude22       [30] = "/Magnitude/22.out";
  char Magnitude23       [30] = "/Magnitude/23.out";
  char Maximum00         [30] = "/Maximum/00.out";
  char Maximum01         [30] = "/Maximum/01.out";
  char Maximum02         [30] = "/Maximum/02.out";
  char Maximum03         [30] = "/Maximum/03.out";
  char Maximum04         [30] = "/Maximum/04.out";
  char Maximum05         [30] = "/Maximum/05.out";
  char Maximum06         [30] = "/Maximum/06.out";
  char Maximum07         [30] = "/Maximum/07.out";
  char Maximum08         [30] = "/Maximum/08.out";
  char Maximum09         [30] = "/Maximum/09.out";
  char Maximum10         [30] = "/Maximum/10.out";
  char Maximum11         [30] = "/Maximum/11.out";
  char Maximum12         [30] = "/Maximum/12.out";
  char Maximum13         [30] = "/Maximum/13.out";
  char Maximum14         [30] = "/Maximum/14.out";
  char Maximum15         [30] = "/Maximum/15.out";
  char Maximum16         [30] = "/Maximum/16.out";
  char Maximum17         [30] = "/Maximum/17.out";
  
  strcat (GeneralFile,           MO[MM-1]);
  strcat (BesselFile,            MO[MM-1]);
  strcat (GreatestFile,          MO[MM-1]);
  strcat (TrackCentralFile,      MO[MM-1]);
  strcat (UmbralShadowFile,      MO[MM-1]);
  strcat (P1234File,             MO[MM-1]);
  strcat (PenumbralContact1File, MO[MM-1]);
  strcat (PenumbralContact2File, MO[MM-1]);
  strcat (Magnitude00File,       MO[MM-1]);
  strcat (Magnitude01File,       MO[MM-1]);
  strcat (Magnitude02File,       MO[MM-1]);
  strcat (Magnitude03File,       MO[MM-1]);
  strcat (Magnitude04File,       MO[MM-1]);
  strcat (Magnitude05File,       MO[MM-1]);
  strcat (Magnitude06File,       MO[MM-1]);
  strcat (Magnitude07File,       MO[MM-1]);
  strcat (Magnitude08File,       MO[MM-1]);
  strcat (Magnitude09File,       MO[MM-1]);
  strcat (Magnitude10File,       MO[MM-1]);
  strcat (Magnitude11File,       MO[MM-1]);
  strcat (Magnitude12File,       MO[MM-1]);
  strcat (Magnitude13File,       MO[MM-1]);
  strcat (Magnitude14File,       MO[MM-1]);
  strcat (Magnitude15File,       MO[MM-1]);
  strcat (Magnitude16File,       MO[MM-1]);
  strcat (Magnitude17File,       MO[MM-1]);
  strcat (Magnitude18File,       MO[MM-1]);
  strcat (Magnitude19File,       MO[MM-1]);
  strcat (Magnitude20File,       MO[MM-1]);
  strcat (Magnitude21File,       MO[MM-1]);
  strcat (Magnitude22File,       MO[MM-1]);
  strcat (Magnitude23File,       MO[MM-1]);
  strcat (Maximum00File,         MO[MM-1]);
  strcat (Maximum01File,         MO[MM-1]);
  strcat (Maximum02File,         MO[MM-1]);
  strcat (Maximum03File,         MO[MM-1]);
  strcat (Maximum04File,         MO[MM-1]);
  strcat (Maximum05File,         MO[MM-1]);
  strcat (Maximum06File,         MO[MM-1]);
  strcat (Maximum07File,         MO[MM-1]);
  strcat (Maximum08File,         MO[MM-1]);
  strcat (Maximum09File,         MO[MM-1]);
  strcat (Maximum10File,         MO[MM-1]);
  strcat (Maximum11File,         MO[MM-1]);
  strcat (Maximum12File,         MO[MM-1]);
  strcat (Maximum13File,         MO[MM-1]);
  strcat (Maximum14File,         MO[MM-1]);
  strcat (Maximum15File,         MO[MM-1]);
  strcat (Maximum16File,         MO[MM-1]);
  strcat (Maximum17File,         MO[MM-1]);

  strcat (GeneralFile,           General);
  strcat (BesselFile,            Bessel);
  strcat (GreatestFile,          Greatest);
  strcat (TrackCentralFile,      TrackCentral);
  strcat (UmbralShadowFile,      UmbralShadow);
  strcat (P1234File,             P1234);
  strcat (PenumbralContact1File, PenumbralContact1);
  strcat (PenumbralContact2File, PenumbralContact2);
  strcat (Magnitude00File,       Magnitude00);
  strcat (Magnitude01File,       Magnitude01);
  strcat (Magnitude02File,       Magnitude02);
  strcat (Magnitude03File,       Magnitude03);
  strcat (Magnitude04File,       Magnitude04);
  strcat (Magnitude05File,       Magnitude05);
  strcat (Magnitude06File,       Magnitude06);
  strcat (Magnitude07File,       Magnitude07);
  strcat (Magnitude08File,       Magnitude08);
  strcat (Magnitude09File,       Magnitude09);
  strcat (Magnitude10File,       Magnitude10);
  strcat (Magnitude11File,       Magnitude11);
  strcat (Magnitude12File,       Magnitude12);
  strcat (Magnitude13File,       Magnitude13);
  strcat (Magnitude14File,       Magnitude14);
  strcat (Magnitude15File,       Magnitude15);
  strcat (Magnitude16File,       Magnitude16);
  strcat (Magnitude17File,       Magnitude17);
  strcat (Magnitude18File,       Magnitude18);
  strcat (Magnitude19File,       Magnitude19);
  strcat (Magnitude20File,       Magnitude20);
  strcat (Magnitude21File,       Magnitude21);
  strcat (Magnitude22File,       Magnitude22);
  strcat (Magnitude23File,       Magnitude23);
  strcat (Maximum00File,         Maximum00);
  strcat (Maximum01File,         Maximum01);
  strcat (Maximum02File,         Maximum02);
  strcat (Maximum03File,         Maximum03);
  strcat (Maximum04File,         Maximum04);
  strcat (Maximum05File,         Maximum05);
  strcat (Maximum06File,         Maximum06);
  strcat (Maximum07File,         Maximum07);
  strcat (Maximum08File,         Maximum08);
  strcat (Maximum09File,         Maximum09);
  strcat (Maximum10File,         Maximum10);
  strcat (Maximum11File,         Maximum11);
  strcat (Maximum12File,         Maximum12);
  strcat (Maximum13File,         Maximum13);
  strcat (Maximum14File,         Maximum14);
  strcat (Maximum15File,         Maximum15);
  strcat (Maximum16File,         Maximum16);
  strcat (Maximum17File,         Maximum17);
}
