// ########################################################################################
// Program to calculate details of solar eclipses in given year.
// Program employs VSOP87 model of planetary motion, and ELP2000-82B model of lunar motion.
// ########################################################################################

#include <stdio.h>
#include <math.h>
#include <stdlib.h>
#include <string.h>
#include "../../Utilities/Constants.h"
#include "../../Utilities/Time.h"
#include "../../Utilities/Frames.h"
#include "../../Utilities/VSOP_ELP.h"
#include "../../Utilities/Ecliptic.h"
#include "../../Utilities/Equatorial.h"
#include "../../Utilities/File.h"

// ##################
// Control parameters
// ##################

#define TOL        1.e-8   // Iteration tolerace
#define MAXIT      50      // Maximum number of iterations
#define MAXITER    1000    // Maximum number of root finding iteractions
#define ERR        1.e-15  // Root finding tolerance
#define CENTRAL    1000    // Number of points on central line
#define UMBRA      20      // Number of umbral shadows on central line
#define UMBRAPOINT 180     // Number of points on umbral shadow
#define MAXIMUM    250     // Number of points on magnitude = maximum curves

// ###################
// Function prototypes
// ###################

// ...........
// In Moon.cpp
// ...........
// Function to return geocentric elongation of Moon from Sun
double MoonElongation (double t);
// Function to determine dates of all new moons in given year 
void NewMoon (int Y);

// .............
// In Bessel.cpp
// .............
// Function to calculate Besselian elements of solar eclipse
void GetBesselElements (double t, double& x, double& y, double& d, double& f1, double&f2, double& l1, double& l2, double& mu);
// Function to calculate derivatives of Besselian elements
void GetBesselElementDerivatives (double t, double &xdot, double &ydot, double &mudot, double &ddot, double &l1dot, double &l2dot);
// Function to produce table of Besselian elements for total/annular solar eclipse
void WriteBesselElementsTable (int Y, int M, int d, int h, double tanf1, double tanf2, double mudot, char filename[]);

// ..............
// In Eclipse.cpp
// ..............
// Function to determine whether solar eclipse occurs at given new moon
void TestEclipse (double t0);
// Function to return latitude and longitude of point on central line of eclipse
void GetLatituteLongitude (double x, double y, double d, double mu,  
				    double &B, double& zeta, double &u, double &phi, double &H, double &lambda);
// Function to return L1 and L2
void GetL1L2 (double f1, double f2, double l1, double l2, double zeta, double &L1, double &L2);
// Function to return solar altitude at given time and place on Earth's surface
void GetSolarAltitude (double t, double phi, double lambda, double &Sun_Alt);
// Function to return eclipse magnitude at point on central line of eclipse
void GetCentralEclipseMagnitude (double L1, double L2, double &Mg);
// Function to return eclipse duration at point on central line of eclipse
void GetCentralEclipseDuration (double x, double y, double d, double f2, double xdot, double ydot, double mudot, double ddot, double l2dot, 
				double zeta, double L2, double &tau);
// Function to construct filenames
void ConstructFilenames (int MM, const char *MO[],
			 char GeneralFile[],           char BesselFile[],       char GreatestFile[],
			 char TrackCentralFile[],      char UmbralShadowFile[], char P1234File[],
			 char PenumbralContact1File[], char PenumbralContact2File[],
			 char Magnitude00File[],       char Magnitude01File[],
			 char Magnitude02File[],       char Magnitude03File[],
			 char Magnitude04File[],       char Magnitude05File[],
			 char Magnitude06File[],       char Magnitude07File[],
			 char Magnitude08File[],       char Magnitude09File[],
			 char Magnitude10File[],       char Magnitude11File[],
			 char Magnitude12File[],       char Magnitude13File[],
			 char Magnitude14File[],       char Magnitude15File[],
			 char Magnitude16File[],       char Magnitude17File[],
			 char Magnitude18File[],       char Magnitude19File[],
			 char Magnitude20File[],       char Magnitude21File[],
			 char Magnitude22File[],       char Magnitude23File[],
			 char Maximum00File[],         char Maximum01File[],
			 char Maximum02File[],         char Maximum03File[],
			 char Maximum04File[],         char Maximum05File[],
			 char Maximum06File[],         char Maximum07File[],
			 char Maximum08File[],         char Maximum09File[],
			 char Maximum10File[],         char Maximum11File[],
			 char Maximum12File[],         char Maximum13File[],
			 char Maximum14File[],         char Maximum15File[],
			 char Maximum16File[],         char Maximum17File[]);
			 
// ............
// In Delta.cpp
// ............
// Function to calculate gammaDot
double GetgammaDot (double t);
// Function to calculate Delta
double GetDelta (double t);
// Function to calculate Delta
double GetDelta (double x, double y, double d, double l, double f, double alpha);
// Function to calculate DeltaDotDot
double GetDeltaDotDot (double t);
// Function to calculate Delta_max
double GetDelta_max (double t, int flag);
// Function to calculate Delta_min
double GetDelta_min (double t, int flag);
// Function to calculate alpha_max
double Getalpha_max (double x, double y, double d, double L);
// Function to calculate alpha_max
double Getalpha_min (double x, double y, double d, double L);
// Function to calculate alpha
void GetAlpha (double alpha_min, double alpha_max, double x, double y, double d, double l, double f, double &alpha);

// ...........
// In Time.cpp
// ...........
// Function to calculate time of greatest eclipse
void GetCentralTime (double &t);
// Function to calculate start time of central eclipse
void GetCentralStartTime (double &t);
// Function to calculate end time of central eclipse
void GetCentralEndTime (double &t);
// Function to calculate U1/P1 time
void GetUP1 (double &t, int flag);
// Function to calculate U2/P2 time
void GetUP2 (double &t, int flag);
// Function to calculate U3/P3 time
void GetUP3 (double &t, int flag);
// Function to calculate U4/P4 time
void GetUP4 (double &t, int flag);

// ............
// In Track.cpp
// ............
// Function to track central line of total/annular solar eclipse
void TrackCentralLine (double t0, char filename[], char filename1[], char filename2[]);
// Function to track penumbral contact curve
void TrackPenumbralContact (double t, char filename[], char filename1[], char filename2[]);

// ................
// In Magnitude.cpp
// ................
// Function to track points of given eclipse magnitude at maximum eclipse
void TrackMagnitude (double t0, double P1, double P4,
		     char filename00[],  char filename01[], char filename02[], char filename03[],
		     char filename04[],  char filename05[], char filename06[], char filename07[],
		     char filename08[],  char filename09[], char filename10[], char filename11[],
		     char filename12[],  char filename13[], char filename14[], char filename15[],
		     char filename16[],  char filename17[], char filename18[], char filename19[],
		     char filename20[],  char filename21[], char filename22[], char filename23[]);
// Function to calculate magnitude = maximum curves at fixed magnitude
void GetMagnitudeCurves (double t0, double P1, double P4, double Magnitude, int flag1, int I, double dt,
			 FILE *file00, FILE *file01, FILE *file02, FILE *file03);
// Function to calculate magnitude = maximum curve at fixed magnitude
void GetMagnitudeCurve (double P1, double P4, double tstart, double tend, double Magnitude, int flag, int flag1, int I, double dt,
			FILE *file);
// Function to calculate latitude and longitude of given eclipse magnitude curve at given time at maximum eclipse
void GetMagnitudeLatitudeLongitude (double t, double Magnitude, int flag, int flag1,
				    double &alpha, double &xi, double &eta, double &R, double &B, double &phi, double &lambda);

// ..............
// In Maximum.cpp
// ..............
// Function to track points of maximum eclipse at given times
void TrackMaximum (double t0, double P1, double P4, double Mg, 
		   char filename00[],  char filename01[], char filename02[], char filename03[],
		   char filename04[],  char filename05[], char filename06[], char filename07[],
		   char filename08[],  char filename09[], char filename10[], char filename11[],
		   char filename12[],  char filename13[], char filename14[], char filename15[],
		   char filename16[],  char filename17[]);
// Function to calculate magnitude = maximum curves at fixed time
void GetMaximumCurves (double t, double Mg, int I, FILE *file00, FILE *file01);
// Function to calculate magnitude = maximum curve at fixed time
void GetMaximumCurve (double t, double Mg, int flag, int I, FILE *file);
