// Time.cpp

#include "Eclipse.h"

// ==============================================
// Function to calculate time of greatest eclipse
// ==============================================
void GetCentralTime (double &t)
{
  // Search for zero of gammaDot via bisection/Regula Falsi method
  double t0 = t - 1. /24.;
  double t1 = t + 1. /24.;
  double t2 = (t0 + t1) /2.;

  double t3; int count = 0;
  do
    {
      if (GetgammaDot (t0) * GetgammaDot (t2) < 0.)
	t1 = t2;
      else
	t0 = t2;

      if (count < 10)
	t3 = (t0 + t1) /2.;
      else
	t3 = t0 - ((t1 - t0) / (GetgammaDot (t1) - GetgammaDot (t0))) * GetgammaDot (t0);

      if (fabs (t3 - t2) < ERR)
	break;

       if (count > MAXITER)
	 {
	   printf ("Warning: Root finding failed to converge in GetCentralTime\n");
	   break;
	 }
 
      t2 = t3;
      count++;
    }
  while (1);

  t = t3;
}

// ===================================================
// Function to calculate start time of central eclipse
// ===================================================
void GetCentralStartTime (double &t)
{
  // Search for zero of Delta via bisection/Regula Falsi method
  double D   = GetDelta       (t);
  double Dtt = GetDeltaDotDot (t);
  double Dt  = sqrt (- 2. * D /Dtt);
  
  double t0 = t;
  double t1 = t - 2.*Dt;
  double t2 = (t0 + t1) /2.;

  double t3; int count = 0;
  do
    {
      if (GetDelta (t0) * GetDelta (t2) < 0.)
	t1 = t2;
      else
	t0 = t2;

      if (count < 10)
	t3 = (t0 + t1) /2.;
      else
	t3 = t0 - ((t1 - t0) / (GetDelta (t1) - GetDelta (t0))) * GetDelta (t0);

      if (fabs (t3 - t2) < ERR)
	break;

      if (count > MAXITER)
	 {
	   printf ("Warning: Root finding failed to converge in GetStartTime\n");
	   break;
	 }
  
      t2 = t3;
      count++;
    }
  while (1);

  t = t3;
}

// =================================================
// Function to calculate end time of central eclipse
// =================================================
void GetCentralEndTime (double &t)
{
  // Search for zero of Delta via bisection/Regula Falsi method
  double D   = GetDelta       (t);
  double Dtt = GetDeltaDotDot (t);
  double Dt  = sqrt (- 2. * D /Dtt);
  
  double t0 = t;
  double t1 = t + 2.*Dt;
  double t2 = (t0 + t1) /2.;

  double t3; int count = 0;
  do
    {
      if (GetDelta (t0) * GetDelta (t2) < 0.)
	t1 = t2;
      else
	t0 = t2;

      if (count < 10)
	t3 = (t0 + t1) /2.;
      else
	t3 = t0 - ((t1 - t0) / (GetDelta (t1) - GetDelta (t0))) * GetDelta (t0);

      if (fabs (t3 - t2) < ERR)
	break;

      if (count > MAXITER)
	 {
	   printf ("Warning: Root finding failed to converge in GetStartTime\n");
	   break;
	 }
  
      t2 = t3;
      count++;
    }
  while (1);

  t = t3;
}

// ================================
// Function to calculate U1/P1 time
// flag = 0/1 for U1/P1
// ================================
void GetUP1 (double &t, int flag)
{
  // Search for zero of Delta_max via bisection/Regula Falsi method
  double D   = GetDelta_max   (t, flag);
  double Dtt = GetDeltaDotDot (t);
  double Dt  = sqrt (- 2. * D /Dtt);
  
  double t0 = t;
  double t1 = t - 2.*Dt;
  double t2 = (t0 + t1) /2.;

  double t3; int count = 0;
  do
    {
      if (GetDelta_max (t0, flag) * GetDelta_max (t2, flag) < 0.)
	t1 = t2;
      else
	t0 = t2;

      if (count < 10)
	t3 = (t0 + t1) /2.;
      else
	t3 = t0 - ((t1 - t0) / (GetDelta_max (t1, flag) - GetDelta_max (t0, flag))) * GetDelta_max (t0, flag);

      if (fabs (t3 - t2) < ERR)
	break;

      if (count > MAXITER)
	 {
	   printf ("Warning: Root finding failed to converge in UP1\n");
	   break;
	 }
  
      t2 = t3;
      count++;
    }
  while (1);

  t = t3;
}

// ================================
// Function to calculate U2/P2 time
// flag = 0/1 for U2/P2
// ================================
void GetUP2 (double &t, int flag)
{
  // Search for zero of Delta_max via bisection/Regula Falsi method
  double D = GetDelta_min (t, flag);
  if (D < 0.)
    {
      t = - 1.e15;
    }
  else
    {
      double Dtt = GetDeltaDotDot (t);
      double Dt  = sqrt (- 2. * D /Dtt);
      
      double t0 = t;
      double t1 = t - 2.*Dt;
      double t2 = (t0 + t1) /2.;
      
      double t3; int count = 0;
      do
	{
	  if (GetDelta_min (t0, flag) * GetDelta_min (t2, flag) < 0.)
	    t1 = t2;
	  else
	    t0 = t2;
	  
	  if (count < 10)
	    t3 = (t0 + t1) /2.;
	  else
	    t3 = t0 - ((t1 - t0) / (GetDelta_min (t1, flag) - GetDelta_min (t0, flag))) * GetDelta_min (t0, flag);
	  
	  if (fabs (t3 - t2) < ERR)
	    break;
	  
	  if (count > MAXITER)
	    {
	      printf ("Warning: Root finding failed to converge in UP2\n");
	      break;
	    }

	  t2 = t3;
	  count++;
	}
      while (1);
      
      t = t3;
    }
}

// ================================
// Function to calculate U3/P3 time 
// flag = 0/1 for U2/P2
// ================================
void GetUP3 (double &t, int flag)
{
  // Search for zero of Delta_max via bisection/Regula Falsi method
  double D = GetDelta_min (t, flag);
  if (D < 0.)
    {
      t = - 1.e15;
    }
  else
    {
      double Dtt = GetDeltaDotDot (t);
      double Dt  = sqrt (- 2. * D /Dtt);
      
      double t0 = t;
      double t1 = t + 2.*Dt;
      double t2 = (t0 + t1) /2.;
      
      double t3; int count = 0;
      do
	{
	  if (GetDelta_min (t0, flag) * GetDelta_min (t2, flag) < 0.)
	    t1 = t2;
	  else
	    t0 = t2;
	  
	  if (count < 10)
	    t3 = (t0 + t1) /2.;
	  else
	    t3 = t0 - ((t1 - t0) / (GetDelta_min (t1, flag) - GetDelta_min (t0, flag))) * GetDelta_min (t0, flag);
	  
	  if (fabs (t3 - t2) < ERR)
	    break;
	  
	  if (count > MAXITER)
	    {
	      printf ("Warning: Root finding failed to converge in UP3\n");
	      break;
	    }
	  
	  t2 = t3;
	  count++;
	}
      while (1);
      
      t = t3;
    }
}

// ================================
// Function to calculate U4/P4 time
// flag = 0/1 for U4/P4
// ================================
void GetUP4 (double &t, int flag)
{
  // Search for zero of Delta_max via bisection/Regula Falsi method
  double D   = GetDelta_max   (t, flag);
  double Dtt = GetDeltaDotDot (t);
  double Dt  = sqrt (- 2. * D /Dtt);
  
  double t0 = t;
  double t1 = t + 2.*Dt;
  double t2 = (t0 + t1) /2.;

  double t3; int count = 0;
  do
    {
      if (GetDelta_max (t0, flag) * GetDelta_max (t2, flag) < 0.)
	t1 = t2;
      else
	t0 = t2;

      if (count < 10)
	t3 = (t0 + t1) /2.;
      else
	t3 = t0 - ((t1 - t0) / (GetDelta_max (t1, flag) - GetDelta_max (t0, flag))) * GetDelta_max (t0, flag);

      if (fabs (t3 - t2) < ERR)
	break;

      if (count > MAXITER)
	 {
	   printf ("Warning: Root finding failed to converge in UP4\n");
	   break;
	 }
  
      t2 = t3;
      count++;
    }
  while (1);

  t = t3;
}

