// Track.cpp

#include "Eclipse.h"

// =============================================================
// Function to track central line of total/annular solar eclipse
// =============================================================
void TrackCentralLine (double t0, char filename[], char filename1[], char filename2[])
{
  // ...............
  // Open data files
  // ...............
  FILE *file  = OpenFile (filename,  "w");
  FILE *file1 = OpenFile (filename1, "w");
  FILE *file2 = OpenFile (filename2, "w");

  // .................................................
  // Output latitude and longitude of greatest eclipse
  // .................................................
  {
    // Get Besselian elements
    double x, y, d, f1, f2, l1, l2, mu;
    GetBesselElements (t0, x, y, d, f1, f2, l1, l2, mu);
    
    // Get latitude and longitude
    double B, zeta, u, phi, H, lambda;
    GetLatituteLongitude (x, y, d, mu, B, zeta, u, phi, H, lambda);

    // Get L2 and L2
    double L1, L2;
    GetL1L2 (f1, f2, l1, l2, zeta, L1, L2);

    // Output data
    fprintf (file, "%e %e %e %e %e\n", (180./M_PI) * phi, (180./M_PI) * lambda, L2, (180./M_PI) * phi, (180./M_PI) * lambda);
  }

  // ..........................................
  // Get start and end times of central eclipse
  // ..........................................
  double tstart = t0, tend = t0;
  GetCentralStartTime (tstart);
  GetCentralEndTime   (tend);

  // .......................................................
  // Track central line at N+1 equally spaced points in time
  // .......................................................
  int N = CENTRAL;
  for (int i = 0; i <= N; i++)
    {
      // Calculate time
      double t = tstart + (tend - tstart) * double (i) /double (N);
      
      // Get Besselian elements
      double x, y, d, f1, f2, l1, l2, mu;
      GetBesselElements (t, x, y, d, f1, f2, l1, l2, mu);

      // Get derivatives of Besselian elements at time of greatest eclipse
      double xdot, ydot, mudot, ddot, l1dot, l2dot;
      GetBesselElementDerivatives (t, xdot, ydot, mudot, ddot, l1dot, l2dot);
      
      // Get latitude and longitude
      double B, zeta, u, phi, H, lambda;
      GetLatituteLongitude (x, y, d, mu, B, zeta, u, phi, H, lambda);

      // Get eclipse magnitude
      double Mg, L1, L2;
      GetL1L2 (f1, f2, l1, l2, zeta, L1, L2);
      GetCentralEclipseMagnitude (L1, L2, Mg);
    
      // Get eclipse duration at greatest eclipse
      double tau;
      GetCentralEclipseDuration (x, y, d, f2, xdot, ydot, mudot, ddot, l2dot, zeta, L2, tau);

      // Get solar altitude
      double Sun_Alt;
      GetSolarAltitude (t, phi, lambda, Sun_Alt);
         
      // Output data
      fprintf (file1, "%e %e %e %e %e %e\n",
	       24. * (t - t0),
	       (180./M_PI) * phi, (180./M_PI) * lambda, 
	       (180./M_PI) * Sun_Alt, Mg, tau);
    }

  // ......................................................
  // Output umbra data at K+1 equally spaced points in time
  // ......................................................
  int K = UMBRA;
  {
    for (int k = 0; k <= K; k++)
      {
	// Calculate time
	double t = tstart + (tend - tstart) * double (k) /double (K);

	// Get Besselian elements of umbra center 
	double x, y, d, f1, f2, l1, l2, mu;
	GetBesselElements (t, x, y, d, f1, f2, l1, l2, mu);
	
	// Get latitude and longitude of umbra center 
	double B, zeta, u, phi, H, lambda;
	GetLatituteLongitude (x, y, d, mu, B, zeta, u, phi, H, lambda);
	
	// Calculate umbra boundary at interval equally spaced points
	double L1, L2, xi, eta; int interval = UMBRAPOINT;
	for (int i = 0; i <= interval; i++)
	  {
	    // Set orientation angle 
	    double alpha = 2.*M_PI * double (i) /double (interval);

	    // Calculate latitude and longitude at umbra boundary
	    for (int i = 0; i < MAX; i++)
	      {
		GetL1L2 (f1, f2, l1, l2, zeta, L1, L2);
		xi  = x + fabs (L2) * cos (alpha);
		eta = y + fabs (L2) * sin (alpha);
		GetZeta (xi, eta, d, B, zeta);
	      }
	    GetLatituteLongitude (xi, eta, d, mu, B, zeta, u, phi, H, lambda);

	    // Get solar altitude at umbra boundary
	    double Sun_Alt;
	    GetSolarAltitude (t, phi, lambda, Sun_Alt);

	    // Output boundary data if umbra boundary visible
	    if (B > ERR && Sun_Alt > 0.)
	      fprintf (file2, "%e %e\n", (180./M_PI) * phi, (180./M_PI) * lambda);
	  }
      }
  }

  // ................
  // Close data files
  // ................
  fclose (file); fclose (file1); fclose (file2);
}

// =======================================================
// Function to track central line of partial solar eclipse
// =======================================================
void TrackCentralLine1 (double t0, double P1, double P4, char filename[], char filename1[], char filename2[])
{
  // ...............
  // Open data files
  // ...............
  FILE *file  = OpenFile (filename,  "w");
  FILE *file1 = OpenFile (filename1, "w");
  FILE *file2 = OpenFile (filename2, "w");

  // .................................................
  // Get latitude and longitude of greatest eclipse
  // .................................................
  double lambda0, phi0, L2;
  {
    // Get Besselian elements
    double x, y, d, f1, f2, l1, l2, mu;
    GetBesselElements (t0, x, y, d, f1, f2, l1, l2, mu);
    
    // Get latitude and longitude
    double B, zeta, u, phi, H, lambda;
    double omega =  1. /sqrt (1. - fe * (2. - fe) * cos (d) * cos (d));
    double fac   = sqrt (x*x + omega*omega * y*y);
    GetLatituteLongitude (x/fac, y/fac, d, mu, B, zeta, u, phi0, H, lambda0);

    // Get L2 and L2
    double L1, L2;
    GetL1L2 (f1, f2, l1, l2, zeta, L1, L2);
  }
  
  // ......................................
  // Get latitude and longitude of P1 point
  // ......................................
  double lambda1, phi1;
  {
    // Get Besselian elements at P1
    double x, y, d, f1, f2, l1, l2, mu;
    GetBesselElements (P1, x, y, d, f1, f2, l1, l2, mu);

    // Calculate alpha
    double alpha = Getalpha_max (x, y, d, l1);
      
    // Recalculate latitude and longitude at P1 to take account of variation in L1
    double xi, eta, B, zeta = 0., H, u, L1, L2;
    for (int i = 0; i < MAX; i++)
      {
	GetL1L2 (f1, f2, l1, l2, zeta, L1, L2);
	alpha = Getalpha_max (x, y, d, L1);
	xi    = x + fabs (L1) * cos (alpha);
	eta   = y + fabs (L1) * sin (alpha);
	GetZeta (xi, eta, d, B, zeta);
      }
    GetLatituteLongitude (xi, eta, d, mu, B, zeta, u, phi1, H, lambda1);
  }

  // .................................................
  // Get latitude and longitude of P4 point
  // .................................................
  double lambda4, phi4;
   {
    // Get Besselian elements at P4
    double x, y, d, f1, f2, l1, l2, mu;
    GetBesselElements (P4, x, y, d, f1, f2, l1, l2, mu);

    // Calculate alpha
    double alpha = Getalpha_max (x, y, d, l1);
      
    // Recalculate latitude and longitude at P4 to take account of variation in L1
    double xi, eta, B, zeta = 0., u, H, L1, L2;
    for (int i = 0; i < MAX; i++)
      {
	GetL1L2 (f1, f2, l1, l2, zeta, L1, L2);
	alpha = Getalpha_max (x, y, d, L1);
	xi    = x + fabs (L1) * cos (alpha);
	eta   = y + fabs (L1) * sin (alpha);
	GetZeta (xi, eta, d, B, zeta);
      }
    GetLatituteLongitude (xi, eta, d, mu, B, zeta, u, phi4, H, lambda4);
   }

  // ...........
  // Output data
  // ...........
  {
    double lambda, phi;
    if (lambda1 > lambda4)
      lambda = (lambda1 + lambda4) /2.;
    else
      lambda = M_PI + (lambda1 + lambda4) /2.;
    phi    = (phi1    + phi4)    /2.;
    
    fprintf (file, "%e %e %e %e %e\n", (180./M_PI) * phi, (180./M_PI) * lambda, L2,  (180./M_PI) * phi0, (180./M_PI) * lambda0);
  }

  // ................
  // Close data files
  // ................
  fclose (file); fclose (file1); fclose (file2);
}

// =========================================
// Function to track penumbral contact curve
// =========================================
void TrackPenumbralContact (double t, char filename[], char filename1[], char filename2[])
{
  // ...............
  // Open data files
  // ...............
  FILE *file  = OpenFile (filename,  "w");
  FILE *file1 = OpenFile (filename1, "w");
  FILE *file2 = OpenFile (filename2, "w");

  // ........................
  // Get umbral contact times
  // ........................
  double P1 = t, P2 = t, P3 = t, P4 = t;
  GetUP1 (P1, 1);
  GetUP2 (P2, 1);
  GetUP3 (P3, 1);
  GetUP4 (P4, 1);

  // Output locations of P1, P2, P3, P4
  double phiP1, lambdaP1, phiP2, lambdaP2, phiP3, lambdaP3, phiP4, lambdaP4;
  {
    // Get Besselian elements at P1
    double x, y, d, f1, f2, l1, l2, mu;
    GetBesselElements (P1, x, y, d, f1, f2, l1, l2, mu);

    // Calculate alpha
    double alpha = Getalpha_max (x, y, d, l1);
 
    // Recalculate latitude and longitude at P1 to take account of variation in L1
    double xi, eta, B, zeta = 0., H, u, L1, L2;
    for (int i = 0; i < MAX; i++)
      {
	GetL1L2 (f1, f2, l1, l2, zeta, L1, L2);
	alpha = Getalpha_max (x, y, d, L1);
	xi    = x + fabs (L1) * cos (alpha);
	eta   = y + fabs (L1) * sin (alpha);
	GetZeta (xi, eta, d, B, zeta);
      }
    GetLatituteLongitude (xi, eta, d, mu, B, zeta, u, phiP1, H, lambdaP1);

    fprintf (file, "%e %e\n", (180./M_PI) * phiP1, (180./M_PI) * lambdaP1);
  }

  if (P2 > -1.e14)
    {
      // Get Besselian elements at P2
      double x, y, d, f1, f2, l1, l2, mu;
      GetBesselElements (P2, x, y, d, f1, f2, l1, l2, mu);
      
      // Calculate alpha
      double alpha = Getalpha_min (x, y, d, l1);
      
      // Recalculate latitude and longitude at P2 to take account of variation in L1
      double xi, eta, B, zeta = 0., u, H, L1, L2;
      for (int i = 0; i < MAX; i++)
	{
	  GetL1L2 (f1, f2, l1, l2, zeta, L1, L2);
	  alpha = Getalpha_min (x, y, d, L1);
	  xi    = x + fabs (L1) * cos (alpha);
	  eta   = y + fabs (L1) * sin (alpha);
	  GetZeta (xi, eta, d, B, zeta);
	}
      GetLatituteLongitude (xi, eta, d, mu, B, zeta, u, phiP2, H, lambdaP2);
      
      fprintf (file, "%e %e\n", (180./M_PI) * phiP2, (180./M_PI) * lambdaP2);
    }

  if (P3 > -1.e14)
    {
      // Get Besselian elements at P3
      double x, y, d, f1, f2, l1, l2, mu;
      GetBesselElements (P3, x, y, d, f1, f2, l1, l2, mu);
      
      // Calculate alpha
      double alpha = Getalpha_min (x, y, d, l1);
      
      // Recalculate latitude and longitude at P3 to take account of variation in L1
      double xi, eta, B, zeta = 0., u, H, L1, L2;
      for (int i = 0; i < MAX; i++)
	{
	  GetL1L2 (f1, f2, l1, l2, zeta, L1, L2);
	  alpha = Getalpha_min (x, y, d, L1);
	  xi    = x + fabs (L1) * cos (alpha);
	  eta   = y + fabs (L1) * sin (alpha);
	  GetZeta (xi, eta, d, B, zeta);
	}
      GetLatituteLongitude (xi, eta, d, mu, B, zeta, u, phiP3, H, lambdaP3);
 
      fprintf (file, "%e %e\n", (180./M_PI) * phiP3, (180./M_PI) * lambdaP3);
    }
  
  {
    // Get Besselian elements at P4
    double x, y, d, f1, f2, l1, l2, mu;
    GetBesselElements (P4, x, y, d, f1, f2, l1, l2, mu);

    // Calculate alpha
    double alpha = Getalpha_max (x, y, d, l1);
      
    // Recalculate latitude and longitude at P4 to take account of variation in L1
    double xi, eta, B, zeta = 0., u, H, L1, L2;
    for (int i = 0; i < MAX; i++)
      {
	GetL1L2 (f1, f2, l1, l2, zeta, L1, L2);
	alpha = Getalpha_max (x, y, d, L1);
	xi    = x + fabs (L1) * cos (alpha);
	eta   = y + fabs (L1) * sin (alpha);
	GetZeta (xi, eta, d, B, zeta);
      }
    GetLatituteLongitude (xi, eta, d, mu, B, zeta, u, phiP4, H, lambdaP4);

    fprintf (file, "%e %e\n", (180./M_PI) * phiP4, (180./M_PI) * lambdaP4);
   }

  // .............................
  // Track penumbral contact curve
  // .............................
  if (P2 > -1.e14)
    {
      // Output P1 limit of western curve
      fprintf (file1, "%e %e %e %e\n", (180./M_PI) * phiP1, (180./M_PI) * lambdaP1, (180./M_PI) * phiP1, (180./M_PI) * lambdaP1);

      // Find I-1 equally spaced (in time) points on western contact curve
      int I = CENTRAL;
      for (int i = 1; i < I; i++)
	{
	  // Set time
	  double t = P1 + (P2 - P1) * double (i) / double (I);

	  // Get Besselian elements
	  double x, y, d, f1, f2, l1, l2, mu;
	  GetBesselElements (t, x, y, d, f1, f2, l1, l2, mu);

	  // Calculate alpha_min and alpha_max
	  double alpha_min = Getalpha_min (x, y, d, l1);
	  double alpha_max = Getalpha_max (x, y, d, l1);
    
	  // Recalculate alpha_max and alpha_min to take into account variation in L1
	  double xi, eta, B, zeta = 0., u, phi, H, lambda, L1, L2;
	  for (int j = 0; j < MAX; j++)
	    {
	      GetL1L2 (f1, f2, l1, l2, zeta, L1, L2);
	      alpha_min = Getalpha_min (x, y, d, L1);
	      xi        = x + fabs (L1) * cos (alpha_min);
	      eta       = y + fabs (L1) * sin (alpha_min);
	      GetZeta (xi, eta, d, B, zeta);
	    }
	  GetLatituteLongitude (xi, eta, d, mu, B, zeta, u, phi, H, lambda);
	  
	  zeta = 0.;
	  for (int j = 0; j < MAX; j++)
	    {
	      GetL1L2 (f1, f2, l1, l2, zeta, L1, L2);
	      alpha_max = Getalpha_max (x, y, d, L1);
	      xi        = x + fabs (L1) * cos (alpha_max);
	      eta       = y + fabs (L1) * sin (alpha_max);
	      GetZeta (xi, eta, d, B, zeta);
	    }
	  GetLatituteLongitude (xi, eta, d, mu, B, zeta, u, phi, H, lambda);

	  // Find first alpha value
	  double alpha1;
	  GetAlpha (alpha_min, alpha_max, x, y, d, l1, f1, alpha1);

	  // Find second alpha value
	  double alpha2;
	  GetAlpha (alpha_max-2.*M_PI, alpha_min, x, y, d, l1, f1, alpha2);

	  // Calculate latitude and longitude
	  double xi1, eta1, B1, zeta1 = 0., u1, L11, L21, phi1, H1, lambda1;
	  double xi2, eta2, B2, zeta2 = 0., u2, L12, L22, phi2, H2, lambda2;
	  for (int j = 0; j < MAX; j++)
	    {
	      GetL1L2 (f1, f2, l1, l2, zeta1, L11, L21);
	      xi1  = x + fabs (L11) * cos (alpha1);
	      eta1 = y + fabs (L11) * sin (alpha1);
	      GetZeta (xi1, eta1, d, B1, zeta1);

	      GetL1L2 (f1, f2, l1, l2, zeta2, L12, L22);
	      xi2  = x + fabs (L12) * cos (alpha2);
	      eta2 = y + fabs (L12) * sin (alpha2);
	      GetZeta (xi2, eta2, d, B2, zeta2);
	    }
	  GetLatituteLongitude (xi1, eta1, d, mu, B1, zeta1, u1, phi1, H1, lambda1);
	  GetLatituteLongitude (xi2, eta2, d, mu, B2, zeta2, u2, phi2, H2, lambda2);

	  // Output data
	  fprintf (file1, "%e %e %e %e\n", (180./M_PI) * phi1, (180./M_PI) * lambda1, (180./M_PI) * phi2, (180./M_PI) * lambda2);
	}

      // Output P2 limit of western curve
      fprintf (file1, "%e %e %e %e\n", (180./M_PI) * phiP2, (180./M_PI) * lambdaP2, (180./M_PI) * phiP2, (180./M_PI) * lambdaP2);

      // Output P3 limit of eastern curve
      fprintf (file2, "%e %e %e %e\n", (180./M_PI) * phiP3, (180./M_PI) * lambdaP3, (180./M_PI) * phiP3, (180./M_PI) * lambdaP3);

      // Find I-1 equally spaced (in time) points on western contact curve
      for (int i = 1; i < I; i++)
	{
	  // Set time
	  double t = P3 + (P4 - P3) * double (i) / double (I);

	  // Get Besselian elements
	  double x, y, d, f1, f2, l1, l2, mu;
	  GetBesselElements (t, x, y, d, f1, f2, l1, l2, mu);

	  // Calculate alpha_min and alpha_max
	  double alpha_min = Getalpha_min (x, y, d, l1);
	  double alpha_max = Getalpha_max (x, y, d, l1);
    
	  // Recalculate alpha_max and alpha_min to take into account variation in L1
	  double xi, eta, B, zeta = 0., u, phi, H, lambda, L1, L2;
	  for (int j = 0; j < MAX; j++)
	    {
	      GetL1L2 (f1, f2, l1, l2, zeta, L1, L2);
	      alpha_min = Getalpha_min (x, y, d, L1);
	      xi        = x + fabs (L1) * cos (alpha_min);
	      eta       = y + fabs (L1) * sin (alpha_min);
	      GetZeta (xi, eta, d, B, zeta);
	    }
	  GetLatituteLongitude (xi, eta, d, mu, B, zeta, u, phi, H, lambda);

	  zeta = 0.;
	  for (int j = 0; j < MAX; j++)
	    {
	      GetL1L2 (f1, f2, l1, l2, zeta, L1, L2);
	      alpha_max = Getalpha_max (x, y, d, L1);
	      xi        = x + fabs (L1) * cos (alpha_max);
	      eta       = y + fabs (L1) * sin (alpha_max);
	      GetZeta (xi, eta, d, B, zeta);
	    }
	  GetLatituteLongitude (xi, eta, d, mu, B, zeta, u, phi, H, lambda);

	  // Find first alpha value
	  double alpha1;
	  GetAlpha (alpha_min, alpha_max, x, y, d, l1, f1, alpha1);

	  // Find second alpha value
	  double alpha2;
	  GetAlpha (alpha_max-2.*M_PI, alpha_min, x, y, d, l1, f1, alpha2);

	  // Calculate latitude and longitude
	  double xi1, eta1, B1, zeta1 = 0., u1, L11, L21,  phi1, H1, lambda1;
	  double xi2, eta2, B2, zeta2 = 0., u2, L12, L22,  phi2, H2, lambda2;
	  for (int j = 0; j < MAX; j++)
	    {
	      GetL1L2 (f1, f2, l1, l2, zeta1, L11, L21);
	      xi1  = x + fabs (L11) * cos (alpha1);
	      eta1 = y + fabs (L11) * sin (alpha1);
	      GetZeta (xi1, eta1, d, B1, zeta1);

	      GetL1L2 (f1, f2, l1, l2, zeta2, L12, L22);
	      xi2  = x + fabs (L12) * cos (alpha2);
	      eta2 = y + fabs (L12) * sin (alpha2);
	      GetZeta (xi2, eta2, d, B2, zeta2);
	    }
	  GetLatituteLongitude (xi1, eta1, d, mu, B1, zeta1, u1, phi1, H1, lambda1);
	  GetLatituteLongitude (xi2, eta2, d, mu, B2, zeta2, u2, phi2, H2, lambda2);

	  // Output data
	  fprintf (file2, "%e %e %e %e\n", (180./M_PI) * phi1, (180./M_PI) * lambda1, (180./M_PI) * phi2, (180./M_PI) * lambda2);
	}

      // Output P4 limit of eastern curve
      fprintf (file2, "%e %e %e %e\n", (180./M_PI) * phiP4, (180./M_PI) * lambdaP4, (180./M_PI) * phiP4, (180./M_PI) * lambdaP4);
    }
  else
    {
      // Output P1 limit of curve
      fprintf (file1, "%e %e %e %e\n", (180./M_PI) * phiP1, (180./M_PI) * lambdaP1, (180./M_PI) * phiP1, (180./M_PI) * lambdaP1);

      // Find I-1 equally spaced (in time) points on contact curve
      int I = CENTRAL;
      for (int i = 1; i < I; i++)
	{
	  // Set time
	  double t = P1 + (P4 - P1) * double (i) / double (I);

	  // Get Besselian elements
	  double x, y, d, f1, f2, l1, l2, mu;
	  GetBesselElements (t, x, y, d, f1, f2, l1, l2, mu);

	  // Calculate alpha_min and alpha_max
	  double alpha_min = Getalpha_min (x, y, d, l1);
	  double alpha_max = Getalpha_max (x, y, d, l1);
    
	  // Recalculate alpha_max and alpha_min to take into account variation in L1
	  double xi, eta, B, zeta = 0., u, phi, H, lambda, L1, L2;
	  for (int j = 0; j < MAX; j++)
	    {
	      GetL1L2 (f1, f2, l1, l2, zeta, L1, L2);
	      alpha_min = Getalpha_min (x, y, d, L1);
	      xi        = x + fabs (L1) * cos (alpha_min);
	      eta       = y + fabs (L1) * sin (alpha_min);
	      GetZeta (xi, eta, d, B, zeta);
	    }
	  GetLatituteLongitude (xi, eta, d, mu, B, zeta, u, phi, H, lambda);

	  zeta = 0.;
	  for (int j = 0; j < MAX; j++)
	    {
	      GetL1L2 (f1, f2, l1, l2, zeta, L1, L2);
	      alpha_max = Getalpha_max (x, y, d, L1);
	      xi        = x + fabs (L1) * cos (alpha_max);
	      eta       = y + fabs (L1) * sin (alpha_max);
	      GetZeta (xi, eta, d, B, zeta);
	    }
	  GetLatituteLongitude (xi, eta, d, mu, B, zeta, u, phi, H, lambda);
	      
	  // Find first alpha value
	  double alpha1;
	  GetAlpha (alpha_min, alpha_max, x, y, d, l1, f1, alpha1);

	  // Find second alpha value
	  double alpha2;
	  GetAlpha (alpha_max-2.*M_PI, alpha_min, x, y, d, l1, f1, alpha2);

	  // Calculate latitude and longitude
	  double xi1, eta1, B1, zeta1 = 0., u1, L11, L21, phi1, H1, lambda1;
	  double xi2, eta2, B2, zeta2 = 0., u2, L12, L22, phi2, H2, lambda2;
	  for (int j = 0; j < MAX; j++)
	    {
	      GetL1L2 (f1, f2, l1, l2, zeta1, L11, L21);
	      xi1  = x + fabs (L11) * cos (alpha1);
	      eta1 = y + fabs (L11) * sin (alpha1);
	      GetZeta (xi1, eta1, d, B1, zeta1);

	      GetL1L2 (f1, f2, l1, l2, zeta2, L12, L22);
	      xi2  = x + fabs (L12) * cos (alpha2);
	      eta2 = y + fabs (L12) * sin (alpha2);
	      GetZeta (xi2, eta2, d, B2, zeta2);
	    }
	  GetLatituteLongitude (xi1, eta1, d, mu, B1, zeta1, u1, phi1, H1, lambda1);
	  GetLatituteLongitude (xi2, eta2, d, mu, B2, zeta2, u2, phi2, H2, lambda2);

	  // Output data
	  fprintf (file1, "%e %e %e %e\n", (180./M_PI) * phi1, (180./M_PI) * lambda1, (180./M_PI) * phi2, (180./M_PI) * lambda2);
	}

      // Output P4 limit of curve
      fprintf (file1, "%e %e %e %e\n", (180./M_PI) * phiP4, (180./M_PI) * lambdaP4, (180./M_PI) * phiP4, (180./M_PI) * lambdaP4);
    }
  
  // ................
  // Close data files
  // ................
  fclose (file); fclose (file1); fclose (file2);
}





