// ###########################################
// Definitions of frame of reference functions
// ###########################################

#include "Frames.h"

// ==============================================
// Function to correct for precession of equinox.
//
// t - Julian day number
// B - geocentric ecliptic latitude (radian)
// L - geocentric ecliptic longitude (radian)
//
// See Meeus, pp. 136, 137.
// =============================================
void Precession (double t, double& B, double& L)
{
  double T  = JD_to_100 (t);
  double B0 = B;
  double L0 = L;

  double eta = (M_PI/180./3600.) * (47.0029 * T - 0.003302 * T*T + 0.000060 * T*T*T);
  double Pi  = (M_PI/180.) * 174.876384 + (M_PI/180./3600.) * (- 869.8089 * T + 0.03536 * T*T);
  double p   = (M_PI/180./3600.) * (5029.0966 * T + 1.11113 * T*T - 0.000006 * T*T*T);

  double Ap = cos (eta) * cos (B0) * sin (Pi - L0) - sin (eta) * sin (B0);
  double Bp = cos (B0)  * cos (Pi - L0);
  double Cp = cos (eta) * sin (B0) + sin (eta) * cos (B0) * sin (Pi - L0);

  L = p + Pi - atan2 (Ap, Bp);
  B = asin (Cp);
}

// ============================================
// Function to calculate obliquity of ecliptic.
//
// t - Julian day number
// eps - obliquity (radians)
//
// See Meeus, p. 147.
// ============================================
void Obliquity (double t, double& eps)
{
  double T = JD_to_100 (t);
  
  double eps0 = (M_PI/180.) * (23.+26./60.+21.448/3600.)
  + (M_PI/180./3600.) * (- 46.8150 * T - 0.00059 * T*T + 0.001813 * T*T*T);
  
  double Deps;
  NutationObliquity (t, Deps);
  
  eps = eps0 + Deps;  
}

// ============================================
// Function to calculate nutation in longitude.
// 
// t    - Julian day number
// Dpsi - nutation in longitude (radian)
//
// See Meeus, pp. 143, 144.
// ============================================
void NutationLongitude (double t, double& Dpsi)
{
  double T = JD_to_100 (t);

  double D = (M_PI/180.) * (297.85036 + 445267.111480 * T - 0.0019142 * T*T
			    + T*T*T /189474.);
  double M = (M_PI/180.) * (357.52772 + 35999.050430 * T - 0.0001603 * T*T
			    - T*T*T /300000.);
  double Mp = (M_PI/180.) * (134.96298 + 477198.867398 * T + 0.0086972 * T*T
			     + T*T*T /56250.);
  double F = (M_PI/180.) * (93.27191 + 483202.017538 * T - 0.0036825 * T*T
			    + T*T*T /327270.);
  double O = (M_PI/180.) * (125.04452 - 1934.136261 * T + 0.0020708 * T*T 
			    + T*T*T /450000.);

  D  = atan2 (sin (D),  cos (D));
  M  = atan2 (sin (M),  cos (M));
  Mp = atan2 (sin (Mp), cos (Mp));
  F  = atan2 (sin (F),  cos (F));
  O  = atan2 (sin (O),  cos (O));
  
  int    d[63], m[63], mp[63], f[63], o[63];
  double a0[63], a1[63];
  FILE* file = OpenFile ("../../Utilities/Nutation/DPsi.data", "r");
    if (file == NULL)
    {
      printf ("Error opening DPsi.data\n");
      exit (1);
    }
  for (int i = 0; i < 63; i++)
    fscanf (file, "%d %d %d %d %d %lf %lf", &d[i], &m[i], &mp[i], &f[i], &o[i], &a0[i], &a1[i]);
  fclose (file);
  
  double sum = 0.;
  for (int i = 0; i < 63; i++)
    sum += (a0[i] + a1[i] * T) * sin (d[i]*D + m[i]*M + mp[i]*Mp + f[i]*F + o[i]*O);
  Dpsi = (M_PI/180.) * (0.0001/3600.) * sum;
}

// ============================================
// Function to calculate nutation in obliquity.
// 
// t    - Julian day number
// Deps - nutation in obliquity (radian)
//
// See Meeus, pp. 143, 144.
// ============================================
void NutationObliquity (double t, double& Deps)
{
  double T = JD_to_100 (t);

  double D = (M_PI/180.) * (297.85036 + 445267.111480 * T - 0.0019142 * T*T
			    + T*T*T /189474.);
  double M = (M_PI/180.) * (357.52772 + 35999.050430 * T - 0.0001603 * T*T
			    - T*T*T /300000.);
  double Mp = (M_PI/180.) * (134.96298 + 477198.867398 * T + 0.0086972 * T*T
			     + T*T*T /56250.);
  double F = (M_PI/180.) * (93.27191 + 483202.017538 * T - 0.0036825 * T*T
			    + T*T*T /327270.);
  double O = (M_PI/180.) * (125.04452 - 1934.136261 * T + 0.0020708 * T*T 
			    + T*T*T /450000.);
  
  D  = atan2 (sin (D),  cos (D));
  M  = atan2 (sin (M),  cos (M));
  Mp = atan2 (sin (Mp), cos (Mp));
  F  = atan2 (sin (F),  cos (F));
  O  = atan2 (sin (O),  cos (O));
  
  int    d[63], m[63], mp[63], f[63], o[63];
  double a0[63], a1[63];
  FILE *file = OpenFile ("../../Utilities/Nutation/Deps.data", "r");
  if (file == NULL)
    {
      printf ("Error opening Deps.data\n");
      exit (1);
    }
  for (int i = 0; i < 38; i++)
    fscanf (file, "%d %d %d %d %d %lf %lf", &d[i], &m[i], &mp[i], &f[i], &o[i], &a0[i], &a1[i]);
  fclose (file);
  
  double sum = 0.;
  for (int i = 0; i < 38; i++)
    sum += (a0[i] + a1[i] * T) * cos (d[i]*D + m[i]*M + mp[i]*Mp + f[i]*F + o[i]*O);
  Deps = (M_PI/180.) * (0.0001/3600.) * sum;
}

// ===========================================
// Function to transform into FK5 system.
//
// t  - Julian day number
// L  - geocentric ecliptic longitude (radian)
// B  - geocentric ecliptic latitude (radian)
//
// See Meeus, Eq. (32.3).
// ===========================================
void FK5Correction (double t, double& L, double& B)
{
  double T  = JD_to_100 (t);
  double LL = L - (M_PI/180.) * (1.397 * T - 0.00031 * T*T);

  double DL = (M_PI/180./3600.) * (- 0.09033 + 0.03916 * (cos (LL) + sin (LL)) * tan (B));
  double DB = (M_PI/180./3600.) * 0.03916 * (cos (LL) - sin (LL));

  L += DL;
  B += DB;
}
